@@ -31,7 +31,7 @@ import org.joni.ast.BackRefNode;
 import org.joni.ast.CClassNode;
 import org.joni.ast.CTypeNode;
 import org.joni.ast.CallNode;
-import org.joni.ast.ConsAltNode;
+import org.joni.ast.ListNode;
 import org.joni.ast.EncloseNode;
 import org.joni.ast.Node;
 import org.joni.ast.QuantifierNode;
@@ -79,32 +79,32 @@ final class ArrayCompiler extends Compiler {
     }
 
     @Override
-    protected void compileAltNode(ConsAltNode node) {
-        ConsAltNode aln = node;
+    protected void compileAltNode(ListNode node) {
+        ListNode aln = node;
         int len = 0;
 
         do {
-            len += compileLengthTree(aln.car);
-            if (aln.cdr != null) {
+            len += compileLengthTree(aln.value);
+            if (aln.tail != null) {
                 len += OPSize.PUSH + OPSize.JUMP;
             }
-        } while ((aln = aln.cdr) != null);
+        } while ((aln = aln.tail) != null);
 
         int pos = codeLength + len;  /* goal position */
 
         aln = node;
         do {
-            len = compileLengthTree(aln.car);
-            if (aln.cdr != null) {
+            len = compileLengthTree(aln.value);
+            if (aln.tail != null) {
                 regex.requireStack = true;
                 addOpcodeRelAddr(OPCode.PUSH, len + OPSize.JUMP);
             }
-            compileTree(aln.car);
-            if (aln.cdr != null) {
+            compileTree(aln.value);
+            if (aln.tail != null) {
                 len = pos - (codeLength + OPSize.JUMP);
                 addOpcodeRelAddr(OPCode.JUMP, len);
             }
-        } while ((aln = aln.cdr) != null);
+        } while ((aln = aln.tail) != null);
     }
 
     private boolean isNeedStrLenOpExact(int op) {
@@ -120,8 +120,9 @@ final class ArrayCompiler extends Compiler {
         return isNeedStrLenOpExact(op);
     }
 
-    private int selectStrOpcode(int mbLength, int strLength, boolean ignoreCase) {
+    private int selectStrOpcode(int mbLength, int byteLength, boolean ignoreCase) {
         int op;
+        int strLength = (byteLength + mbLength - 1) / mbLength;
 
         if (ignoreCase) {
             switch(strLength) {
@@ -187,8 +188,8 @@ final class ArrayCompiler extends Compiler {
         }
     }
 
-    private int addCompileStringlength(byte[]bytes, int p, int mbLength, int strLength, boolean ignoreCase) {
-        int op = selectStrOpcode(mbLength, strLength, ignoreCase);
+    private int addCompileStringlength(byte[]bytes, int p, int mbLength, int byteLength, boolean ignoreCase) {
+        int op = selectStrOpcode(mbLength, byteLength, ignoreCase);
         int len = OPSize.OPCODE;
 
         if (Config.USE_STRING_TEMPLATES && opTemplated(op)) {
@@ -196,24 +197,24 @@ final class ArrayCompiler extends Compiler {
             len += OPSize.LENGTH + OPSize.INDEX + OPSize.INDEX;
         } else {
             if (isNeedStrLenOpExact(op)) len += OPSize.LENGTH;
-            len += mbLength * strLength;
+            len += byteLength;
         }
         if (op == OPCode.EXACTMBN) len += OPSize.LENGTH;
         return len;
     }
 
     @Override
-    protected final void addCompileString(byte[]bytes, int p, int mbLength, int strLength, boolean ignoreCase) {
-        int op = selectStrOpcode(mbLength, strLength, ignoreCase);
+    protected final void addCompileString(byte[]bytes, int p, int mbLength, int byteLength, boolean ignoreCase) {
+        int op = selectStrOpcode(mbLength, byteLength, ignoreCase);
         addOpcode(op);
 
         if (op == OPCode.EXACTMBN) addLength(mbLength);
 
         if (isNeedStrLenOpExact(op)) {
             if (op == OPCode.EXACTN_IC || op == OPCode.EXACTN_IC_SB) {
-                addLength(mbLength * strLength);
+                addLength(byteLength);
             } else {
-                addLength(strLength);
+                addLength(byteLength / mbLength);
             }
         }
 
@@ -222,7 +223,7 @@ final class ArrayCompiler extends Compiler {
             addInt(p);
             addTemplate(bytes);
         } else {
-            addBytes(bytes, p, mbLength * strLength);
+            addBytes(bytes, p, byteLength);
         }
     }
 
@@ -237,24 +238,23 @@ final class ArrayCompiler extends Compiler {
         byte[]bytes = sn.bytes;
         int prevLen = enc.length(bytes, p, end);
         p += prevLen;
-
-        int slen = 1;
+        int blen = prevLen;
         int rlen = 0;
 
         while (p < end) {
             int len = enc.length(bytes, p, end);
-            if (len == prevLen) {
-                slen++;
+            if (len == prevLen || ambig) {
+                blen += len;
             } else {
-                int r = addCompileStringlength(bytes, prev, prevLen, slen, ambig);
+                int r = addCompileStringlength(bytes, prev, prevLen, blen, ambig);
                 rlen += r;
                 prev = p;
-                slen = 1;
+                blen = len;
                 prevLen = len;
             }
             p += len;
         }
-        int r = addCompileStringlength(bytes, prev, prevLen, slen, ambig);
+        int r = addCompileStringlength(bytes, prev, prevLen, blen, ambig);
         rlen += r;
         return rlen;
     }
@@ -265,13 +265,11 @@ final class ArrayCompiler extends Compiler {
     }
 
     private void addMultiByteCClass(CodeRangeBuffer mbuf) {
-        addLength(mbuf.used);
-        addInts(mbuf.p, mbuf.used);
+        addLength(mbuf.getUsed());
+        addInts(mbuf.getCodeRange(), mbuf.getUsed());
     }
 
     private int compileLengthCClassNode(CClassNode cc) {
-        if (cc.isShare()) return OPSize.OPCODE + OPSize.POINTER;
-
         int len;
         if (cc.mbuf == null) {
             len = OPSize.OPCODE + BitSet.BITSET_SIZE;
@@ -282,24 +280,18 @@ final class ArrayCompiler extends Compiler {
                 len = OPSize.OPCODE + BitSet.BITSET_SIZE;
             }
 
-            len += OPSize.LENGTH + cc.mbuf.used;
+            len += OPSize.LENGTH + cc.mbuf.getUsed();
         }
         return len;
     }
 
     @Override
     protected void compileCClassNode(CClassNode cc) {
-        if (cc.isShare()) { // shared char class
-            addOpcode(OPCode.CCLASS_NODE);
-            addPointer(cc);
-            return;
-        }
-
         if (cc.mbuf == null) {
             if (cc.isNot()) {
-                addOpcode(enc.isSingleByte() ? OPCode.CCLASS_NOT_SB : OPCode.CCLASS_NOT);
+                addOpcode(OPCode.CCLASS_NOT);
             } else {
-                addOpcode(enc.isSingleByte() ? OPCode.CCLASS_SB : OPCode.CCLASS);
+                addOpcode(OPCode.CCLASS);
             }
             addInts(cc.bs.bits, BitSet.BITSET_SIZE); // add_bitset
         } else {
@@ -330,14 +322,22 @@ final class ArrayCompiler extends Compiler {
         switch (cn.ctype) {
         case CharacterType.WORD:
             if (cn.not) {
-                op = enc.isSingleByte() ? OPCode.NOT_WORD_SB : OPCode.NOT_WORD;
+                if (cn.asciiRange) {
+                    op = OPCode.ASCII_NOT_WORD;
+                } else {
+                    op = OPCode.NOT_WORD;
+                }
             } else {
-                op = enc.isSingleByte() ? OPCode.WORD_SB : OPCode.WORD;
+                if (cn.asciiRange) {
+                    op = OPCode.ASCII_WORD;
+                } else {
+                    op = OPCode.WORD;
+                }
             }
             break;
 
         default:
-            newInternalException(ERR_PARSER_BUG);
+            newInternalException(PARSER_BUG);
             return; // not reached
         } // inner switch
         addOpcode(op);
@@ -346,9 +346,9 @@ final class ArrayCompiler extends Compiler {
     @Override
     protected void compileAnyCharNode() {
         if (isMultiline(regex.options)) {
-            addOpcode(enc.isSingleByte() ? OPCode.ANYCHAR_ML_SB : OPCode.ANYCHAR_ML);
+            addOpcode(OPCode.ANYCHAR_ML);
         } else {
-            addOpcode(enc.isSingleByte() ? OPCode.ANYCHAR_SB : OPCode.ANYCHAR);
+            addOpcode(OPCode.ANYCHAR);
         }
     }
 
@@ -532,9 +532,9 @@ final class ArrayCompiler extends Compiler {
             compileTreeNTimes(qn.target, qn.lower);
             if (qn.nextHeadExact != null && !cknOn(ckn)) {
                 if (isMultiline(regex.options)) {
-                    addOpcode(enc.isSingleByte() ? OPCode.ANYCHAR_ML_STAR_PEEK_NEXT_SB : OPCode.ANYCHAR_ML_STAR_PEEK_NEXT);
+                    addOpcode(OPCode.ANYCHAR_ML_STAR_PEEK_NEXT);
                 } else {
-                    addOpcode(enc.isSingleByte() ? OPCode.ANYCHAR_STAR_PEEK_NEXT_SB : OPCode.ANYCHAR_STAR_PEEK_NEXT);
+                    addOpcode(OPCode.ANYCHAR_STAR_PEEK_NEXT);
                 }
                 if (cknOn(ckn)) {
                     addStateCheckNum(ckn);
@@ -545,15 +545,15 @@ final class ArrayCompiler extends Compiler {
             } else {
                 if (isMultiline(regex.options)) {
                     if (cknOn(ckn)) {
-                        addOpcode(enc.isSingleByte() ? OPCode.STATE_CHECK_ANYCHAR_ML_STAR_SB : OPCode.STATE_CHECK_ANYCHAR_ML_STAR);
+                        addOpcode(OPCode.STATE_CHECK_ANYCHAR_ML_STAR);
                     } else {
-                        addOpcode(enc.isSingleByte() ? OPCode.ANYCHAR_ML_STAR_SB : OPCode.ANYCHAR_ML_STAR);
+                        addOpcode(OPCode.ANYCHAR_ML_STAR);
                     }
                 } else {
                     if (cknOn(ckn)) {
-                        addOpcode(enc.isSingleByte() ? OPCode.STATE_CHECK_ANYCHAR_STAR_SB : OPCode.STATE_CHECK_ANYCHAR_STAR);
+                        addOpcode(OPCode.STATE_CHECK_ANYCHAR_STAR);
                     } else {
-                        addOpcode(enc.isSingleByte() ? OPCode.ANYCHAR_STAR_SB : OPCode.ANYCHAR_STAR);
+                        addOpcode(OPCode.ANYCHAR_STAR);
                     }
                 }
                 if (cknOn(ckn)) {
@@ -705,18 +705,18 @@ final class ArrayCompiler extends Compiler {
             compileTreeNTimes(qn.target, qn.lower);
             if (qn.nextHeadExact != null) {
                 if (isMultiline(regex.options)) {
-                    addOpcode(enc.isSingleByte() ? OPCode.ANYCHAR_ML_STAR_PEEK_NEXT_SB : OPCode.ANYCHAR_ML_STAR_PEEK_NEXT);
+                    addOpcode(OPCode.ANYCHAR_ML_STAR_PEEK_NEXT);
                 } else {
-                    addOpcode(enc.isSingleByte() ? OPCode.ANYCHAR_STAR_PEEK_NEXT_SB : OPCode.ANYCHAR_STAR_PEEK_NEXT);
+                    addOpcode(OPCode.ANYCHAR_STAR_PEEK_NEXT);
                 }
                 StringNode sn = (StringNode)qn.nextHeadExact;
                 addBytes(sn.bytes, sn.p, 1);
                 return;
             } else {
                 if (isMultiline(regex.options)) {
-                    addOpcode(enc.isSingleByte() ? OPCode.ANYCHAR_ML_STAR_SB : OPCode.ANYCHAR_ML_STAR);
+                    addOpcode(OPCode.ANYCHAR_ML_STAR);
                 } else {
-                    addOpcode(enc.isSingleByte() ? OPCode.ANYCHAR_STAR_SB : OPCode.ANYCHAR_STAR);
+                    addOpcode(OPCode.ANYCHAR_STAR);
                 }
                 return;
             }
@@ -795,7 +795,7 @@ final class ArrayCompiler extends Compiler {
         int tlen = compileLengthTree(node.target);
         regex.options = prev;
 
-        if (isDynamic(prev ^ node.option)) {
+        if (Config.USE_DYNAMIC_OPTION && isDynamic(prev ^ node.option)) {
             return OPSize.SET_OPTION_PUSH + OPSize.SET_OPTION + OPSize.FAIL + tlen + OPSize.SET_OPTION;
         } else {
             return tlen;
@@ -806,7 +806,7 @@ final class ArrayCompiler extends Compiler {
     protected void compileOptionNode(EncloseNode node) {
         int prev = regex.options;
 
-        if (isDynamic(prev ^ node.option)) {
+        if (Config.USE_DYNAMIC_OPTION && isDynamic(prev ^ node.option)) {
             addOpcodeOption(OPCode.SET_OPTION_PUSH, node.option);
             addOpcodeOption(OPCode.SET_OPTION, prev);
             addOpcode(OPCode.FAIL);
@@ -816,7 +816,7 @@ final class ArrayCompiler extends Compiler {
         compileTree(node.target);
         regex.options = prev;
 
-        if (isDynamic(prev ^ node.option)) {
+        if (Config.USE_DYNAMIC_OPTION && isDynamic(prev ^ node.option)) {
             addOpcodeOption(OPCode.SET_OPTION, prev);
         }
     }
@@ -843,11 +843,14 @@ final class ArrayCompiler extends Compiler {
                 } else {
                     len += node.isRecursion() ? OPSize.MEMORY_END_REC : OPSize.MEMORY_END;
                 }
-            } else { // USE_SUBEXP_CALL
+            } else if (Config.USE_SUBEXP_CALL && node.isRecursion()) { // USE_SUBEXP_CALL
+                len = OPSize.MEMORY_START_PUSH; // or OPSize.MEMORY_START;
+                len += tlen + (bsAt(regex.btMemEnd, node.regNum) ? OPSize.MEMORY_END_PUSH_REC : OPSize.MEMORY_END_REC);
+            } else {
                 if (bsAt(regex.btMemStart, node.regNum)) {
                     len = OPSize.MEMORY_START_PUSH;
                 } else {
-                    len = OPSize.MEMORY_START;
+                    len= OPSize.MEMORY_START;
                 }
                 len += tlen + (bsAt(regex.btMemEnd, node.regNum) ? OPSize.MEMORY_END_PUSH : OPSize.MEMORY_END);
             }
@@ -866,20 +869,23 @@ final class ArrayCompiler extends Compiler {
         case EncloseType.CONDITION:
             len = OPSize.CONDITION;
             if (node.target.getType() == NodeType.ALT) {
-                ConsAltNode x = (ConsAltNode)node.target;
-                tlen = compileLengthTree(x.car); /* yes-node */
+                ListNode x = (ListNode)node.target;
+                tlen = compileLengthTree(x.value); /* yes-node */
                 len += tlen + OPSize.JUMP;
-                if (x.cdr == null) newInternalException(ERR_PARSER_BUG);
-                x = x.cdr;
-                tlen = compileLengthTree(x.car); /* no-node */
+                if (x.tail == null) newInternalException(PARSER_BUG);
+                x = x.tail;
+                tlen = compileLengthTree(x.value); /* no-node */
                 len += tlen;
-                if (x.cdr != null) newSyntaxException(ERR_INVALID_CONDITION_PATTERN);
+                if (x.tail != null) newSyntaxException(INVALID_CONDITION_PATTERN);
             } else {
-                newInternalException(ERR_PARSER_BUG);
+                newInternalException(PARSER_BUG);
             }
             break;
+        case EncloseType.ABSENT:
+            len = OPSize.PUSH_ABSENT_POS + OPSize.ABSENT + tlen + OPSize.ABSENT_END;
+            break;
         default:
-            newInternalException(ERR_PARSER_BUG);
+            newInternalException(PARSER_BUG);
             return 0; // not reached
         } // switch
         return len;
@@ -890,22 +896,20 @@ final class ArrayCompiler extends Compiler {
         int len;
         switch (node.type) {
         case EncloseType.MEMORY:
-            if (Config.USE_SUBEXP_CALL) {
-                if (node.isCalled()) {
-                    regex.requireStack = true;
-                    addOpcode(OPCode.CALL);
-                    node.callAddr = codeLength + OPSize.ABSADDR + OPSize.JUMP;
-                    node.setAddrFixed();
-                    addAbsAddr(node.callAddr);
-                    len = compileLengthTree(node.target);
-                    len += OPSize.MEMORY_START_PUSH + OPSize.RETURN;
-                    if (bsAt(regex.btMemEnd, node.regNum)) {
-                        len += node.isRecursion() ? OPSize.MEMORY_END_PUSH_REC : OPSize.MEMORY_END_PUSH;
-                    } else {
-                        len += node.isRecursion() ? OPSize.MEMORY_END_REC : OPSize.MEMORY_END;
-                    }
-                    addOpcodeRelAddr(OPCode.JUMP, len);
+            if (Config.USE_SUBEXP_CALL && node.isCalled()) {
+                regex.requireStack = true;
+                addOpcode(OPCode.CALL);
+                node.callAddr = codeLength + OPSize.ABSADDR + OPSize.JUMP;
+                node.setAddrFixed();
+                addAbsAddr(node.callAddr);
+                len = compileLengthTree(node.target);
+                len += OPSize.MEMORY_START_PUSH + OPSize.RETURN;
+                if (bsAt(regex.btMemEnd, node.regNum)) {
+                    len += node.isRecursion() ? OPSize.MEMORY_END_PUSH_REC : OPSize.MEMORY_END_PUSH;
+                } else {
+                    len += node.isRecursion() ? OPSize.MEMORY_END_REC : OPSize.MEMORY_END;
                 }
+                addOpcodeRelAddr(OPCode.JUMP, len);
             } // USE_SUBEXP_CALL
 
             if (bsAt(regex.btMemStart, node.regNum)) {
@@ -926,7 +930,14 @@ final class ArrayCompiler extends Compiler {
                 }
                 addMemNum(node.regNum);
                 addOpcode(OPCode.RETURN);
-            } else { // USE_SUBEXP_CALL
+            } else if (Config.USE_SUBEXP_CALL && node.isRecursion()) { // USE_SUBEXP_CALL
+                if (bsAt(regex.btMemEnd, node.regNum)) {
+                    addOpcode(OPCode.MEMORY_END_PUSH_REC);
+                } else {
+                    addOpcode(OPCode.MEMORY_END_REC);
+                }
+                addMemNum(node.regNum);
+            } else {
                 if (bsAt(regex.btMemEnd, node.regNum)) {
                     addOpcode(OPCode.MEMORY_END_PUSH);
                 } else {
@@ -959,25 +970,34 @@ final class ArrayCompiler extends Compiler {
             addOpcode(OPCode.CONDITION);
             addMemNum(node.regNum);
             if (node.target.getType() == NodeType.ALT) {
-                ConsAltNode x = (ConsAltNode)node.target;
-                len = compileLengthTree(x.car); /* yes-node */
-                if (x.cdr == null) newInternalException(ERR_PARSER_BUG);
-                x = x.cdr;
-                int len2 = compileLengthTree(x.car); /* no-node */
-                if (x.cdr != null) newSyntaxException(ERR_INVALID_CONDITION_PATTERN);
-                x = (ConsAltNode)node.target;
+                ListNode x = (ListNode)node.target;
+                len = compileLengthTree(x.value); /* yes-node */
+                if (x.tail == null) newInternalException(PARSER_BUG);
+                x = x.tail;
+                int len2 = compileLengthTree(x.value); /* no-node */
+                if (x.tail != null) newSyntaxException(INVALID_CONDITION_PATTERN);
+                x = (ListNode)node.target;
                 addRelAddr(len + OPSize.JUMP);
-                compileTree(x.car); /* yes-node */
+                compileTree(x.value); /* yes-node */
                 addOpcodeRelAddr(OPCode.JUMP, len2);
-                x = x.cdr;
-                compileTree(x.car); /* no-node */
+                x = x.tail;
+                compileTree(x.value); /* no-node */
             } else {
-                newInternalException(ERR_PARSER_BUG);
+                newInternalException(PARSER_BUG);
             }
             break;
 
+        case EncloseType.ABSENT:
+            regex.requireStack = true;
+            len = compileLengthTree(node.target);
+            addOpcode(OPCode.PUSH_ABSENT_POS);
+            addOpcodeRelAddr(OPCode.ABSENT, len + OPSize.ABSENT_END);
+            compileTree(node.target);
+            addOpcode(OPCode.ABSENT_END);
+            break;
+
         default:
-            newInternalException(ERR_PARSER_BUG);
+            newInternalException(PARSER_BUG);
             break;
         } // switch
     }
@@ -1029,21 +1049,39 @@ final class ArrayCompiler extends Compiler {
         case AnchorType.BEGIN_POSITION:     addOpcode(OPCode.BEGIN_POSITION);       break;
 
         case AnchorType.WORD_BOUND:
-            addOpcode(enc.isSingleByte() ? OPCode.WORD_BOUND_SB : OPCode.WORD_BOUND);
+            if (node.asciiRange) {
+                addOpcode(OPCode.ASCII_WORD_BOUND);
+            } else {
+                addOpcode(OPCode.WORD_BOUND);
+            }
             break;
 
         case AnchorType.NOT_WORD_BOUND:
-            addOpcode(enc.isSingleByte() ? OPCode.NOT_WORD_BOUND_SB : OPCode.NOT_WORD_BOUND);
+            if (node.asciiRange) {
+                addOpcode(OPCode.ASCII_NOT_WORD_BOUND);
+            } else {
+                addOpcode(OPCode.NOT_WORD_BOUND);
+            }
             break;
 
         case AnchorType.WORD_BEGIN:
-            if (Config.USE_WORD_BEGIN_END)
-                addOpcode(enc.isSingleByte() ? OPCode.WORD_BEGIN_SB : OPCode.WORD_BEGIN);
+            if (Config.USE_WORD_BEGIN_END) {
+                if (node.asciiRange) {
+                    addOpcode(OPCode.ASCII_WORD_BEGIN);
+                } else {
+                    addOpcode(OPCode.WORD_BEGIN);
+                }
+            }
             break;
 
         case AnchorType.WORD_END:
-            if (Config.USE_WORD_BEGIN_END)
-                addOpcode(enc.isSingleByte() ? OPCode.WORD_END_SB : OPCode.WORD_END);
+            if (Config.USE_WORD_BEGIN_END) {
+                if (node.asciiRange) {
+                    addOpcode(OPCode.ASCII_WORD_END);
+                } else {
+                    addOpcode(OPCode.WORD_END);
+                }
+            }
             break;
 
         case AnchorType.KEEP:
@@ -1066,10 +1104,10 @@ final class ArrayCompiler extends Compiler {
             break;
 
         case AnchorType.LOOK_BEHIND:
-            addOpcode(enc.isSingleByte() ? OPCode.LOOK_BEHIND_SB : OPCode.LOOK_BEHIND);
+            addOpcode(OPCode.LOOK_BEHIND);
             if (node.charLength < 0) {
                 n = analyser.getCharLengthTree(node.target);
-                if (analyser.returnCode != 0) newSyntaxException(ERR_INVALID_LOOK_BEHIND_PATTERN);
+                if (analyser.returnCode != 0) newSyntaxException(INVALID_LOOK_BEHIND_PATTERN);
             } else {
                 n = node.charLength;
             }
@@ -1083,7 +1121,7 @@ final class ArrayCompiler extends Compiler {
             addOpcodeRelAddr(OPCode.PUSH_LOOK_BEHIND_NOT, len + OPSize.FAIL_LOOK_BEHIND_NOT);
             if (node.charLength < 0) {
                 n = analyser.getCharLengthTree(node.target);
-                if (analyser.returnCode != 0) newSyntaxException(ERR_INVALID_LOOK_BEHIND_PATTERN);
+                if (analyser.returnCode != 0) newSyntaxException(INVALID_LOOK_BEHIND_PATTERN);
             } else {
                 n = node.charLength;
             }
@@ -1093,7 +1131,7 @@ final class ArrayCompiler extends Compiler {
             break;
 
         default:
-            newInternalException(ERR_PARSER_BUG);
+            newInternalException(PARSER_BUG);
         } // switch
     }
 
@@ -1102,19 +1140,19 @@ final class ArrayCompiler extends Compiler {
 
         switch (node.getType()) {
         case NodeType.LIST:
-            ConsAltNode lin = (ConsAltNode)node;
+            ListNode lin = (ListNode)node;
             do {
-                len += compileLengthTree(lin.car);
-            } while ((lin = lin.cdr) != null);
+                len += compileLengthTree(lin.value);
+            } while ((lin = lin.tail) != null);
             break;
 
         case NodeType.ALT:
-            ConsAltNode aln = (ConsAltNode)node;
+            ListNode aln = (ListNode)node;
             int n = 0;
             do {
-                len += compileLengthTree(aln.car);
+                len += compileLengthTree(aln.value);
                 n++;
-            } while ((aln = aln.cdr) != null);
+            } while ((aln = aln.tail) != null);
             len += (OPSize.PUSH + OPSize.JUMP) * (n - 1);
             break;
 
@@ -1160,7 +1198,7 @@ final class ArrayCompiler extends Compiler {
             break;
 
         case NodeType.QTFR:
-            if (Config.USE_COMBINATION_EXPLOSION_CHECK) {
+            if (Config.USE_CEC) {
                 len = compileCECLengthQuantifierNode((QuantifierNode)node);
             } else {
                 len = compileNonCECLengthQuantifierNode((QuantifierNode)node);
@@ -1176,7 +1214,7 @@ final class ArrayCompiler extends Compiler {
             break;
 
         default:
-            newInternalException(ERR_PARSER_BUG);
+            newInternalException(PARSER_BUG);
 
         } //switch
         return len;
@@ -1206,18 +1244,6 @@ final class ArrayCompiler extends Compiler {
         regex.code[offset] = i;
     }
 
-    private void addObject(Object o) {
-        if (regex.operands == null) {
-            regex.operands = new Object[4];
-        } else if (regex.operandLength >= regex.operands.length) {
-            Object[]tmp = new Object[regex.operands.length << 1];
-            System.arraycopy(regex.operands, 0, tmp, 0, regex.operands.length);
-            regex.operands = tmp;
-        }
-        addInt(regex.operandLength);
-        regex.operands[regex.operandLength++] = o;
-    }
-
     private void addBytes(byte[]bytes, int p ,int length) {
         ensure(codeLength + length);
         int end = p + length;
@@ -1255,10 +1281,6 @@ final class ArrayCompiler extends Compiler {
         addInt(num);
     }
 
-    private void addPointer(Object o) {
-        addObject(o);
-    }
-
     private void addOption(int option) {
         addInt(option);
     }
