@@ -25,7 +25,7 @@ import org.joni.ast.BackRefNode;
 import org.joni.ast.CClassNode;
 import org.joni.ast.CTypeNode;
 import org.joni.ast.CallNode;
-import org.joni.ast.ConsAltNode;
+import org.joni.ast.ListNode;
 import org.joni.ast.EncloseNode;
 import org.joni.ast.Node;
 import org.joni.ast.QuantifierNode;
@@ -46,16 +46,16 @@ abstract class Compiler implements ErrorMessages {
         this.enc = regex.enc;
     }
 
-    final void compile() {
+    final void compile(Node root) {
         prepare();
-        compileTree(analyser.root);
+        compileTree(root);
         finish();
     }
 
     protected abstract void prepare();
     protected abstract void finish();
 
-    protected abstract void compileAltNode(ConsAltNode node);
+    protected abstract void compileAltNode(ListNode node);
 
     private void compileStringRawNode(StringNode sn) {
         if (sn.length() <= 0) return;
@@ -74,21 +74,21 @@ abstract class Compiler implements ErrorMessages {
         byte[]bytes = sn.bytes;
         int prevLen = enc.length(bytes, p, end);
         p += prevLen;
-        int slen = 1;
+        int blen = prevLen;
 
         while (p < end) {
             int len = enc.length(bytes, p, end);
-            if (len == prevLen) {
-                slen++;
+            if (len == prevLen || ambig) {
+                blen += len;
             } else {
-                addCompileString(bytes, prev, prevLen, slen, ambig);
+                addCompileString(bytes, prev, prevLen, blen, ambig);
                 prev = p;
-                slen = 1;
+                blen = len;
                 prevLen = len;
             }
             p += len;
         }
-        addCompileString(bytes, prev, prevLen, slen, ambig);
+        addCompileString(bytes, prev, prevLen, blen, ambig);
     }
 
     protected abstract void addCompileString(byte[]bytes, int p, int mbLength, int strLength, boolean ignoreCase);
@@ -107,14 +107,14 @@ abstract class Compiler implements ErrorMessages {
     protected final void compileTree(Node node) {
         switch (node.getType()) {
         case NodeType.LIST:
-            ConsAltNode lin = (ConsAltNode)node;
+            ListNode lin = (ListNode)node;
             do {
-                compileTree(lin.car);
-            } while ((lin = lin.cdr) != null);
+                compileTree(lin.value);
+            } while ((lin = lin.tail) != null);
             break;
 
         case NodeType.ALT:
-            compileAltNode((ConsAltNode)node);
+            compileAltNode((ListNode)node);
             break;
 
         case NodeType.STR:
@@ -150,7 +150,7 @@ abstract class Compiler implements ErrorMessages {
             break;
 
         case NodeType.QTFR:
-            if (Config.USE_COMBINATION_EXPLOSION_CHECK) {
+            if (Config.USE_CEC) {
                 compileCECQuantifierNode((QuantifierNode)node);
             } else {
                 compileNonCECQuantifierNode((QuantifierNode)node);
@@ -172,7 +172,7 @@ abstract class Compiler implements ErrorMessages {
 
         default:
             // undefined node type
-            newInternalException(ERR_PARSER_BUG);
+            newInternalException(PARSER_BUG);
         } // switch
     }
 
