@@ -23,6 +23,7 @@ import java.io.PrintStream;
 
 public interface Config extends org.jcodings.Config {
     final int CHAR_TABLE_SIZE = 256;
+    final boolean USE_NO_INVALID_QUANTIFIER = true;
     final int SCANENV_MEMNODES_SIZE = 8;
 
     final boolean USE_NAMED_GROUP = true;
@@ -32,7 +33,7 @@ public interface Config extends org.jcodings.Config {
 
     final boolean USE_MONOMANIAC_CHECK_CAPTURES_IN_ENDLESS_REPEAT = true; /* /(?:()|())*\2/ */
     final boolean USE_NEWLINE_AT_END_OF_STRING_HAS_EMPTY_LINE = true;     /* /\n$/ =~ "\n" */
-    final boolean USE_WARNING_REDUNDANT_NESTED_REPEAT_OPERATOR = false;
+    final boolean USE_WARNING_REDUNDANT_NESTED_REPEAT_OPERATOR = true;
 
     final boolean CASE_FOLD_IS_APPLIED_INSIDE_NEGATIVE_CCLASS = true;
 
@@ -42,10 +43,12 @@ public interface Config extends org.jcodings.Config {
     final boolean USE_WORD_BEGIN_END = true;                                /* "\<": word-begin, "\>": word-end */
     final boolean USE_POSIX_API_REGION_OPTION = true;                           /* needed for POSIX API support */
     final boolean USE_FIND_LONGEST_SEARCH_ALL_OF_RANGE = true;
-    final boolean USE_COMBINATION_EXPLOSION_CHECK = false;
+    final boolean USE_CEC = false;
+    final boolean USE_DYNAMIC_OPTION = false;
 
     final int NREGION                   = 10;
     final int MAX_BACKREF_NUM           = 1000;
+    final int MAX_CAPTURE_GROUP_NUM     = 32767;
     final int MAX_REPEAT_NUM            = 100000;
     final int MAX_MULTI_BYTE_RANGES_NUM = 10000;
 
@@ -55,7 +58,6 @@ public interface Config extends org.jcodings.Config {
     // internal config
     final boolean USE_PARSE_TREE_NODE_RECYCLE       = true;
     final boolean USE_OP_PUSH_OR_JUMP_EXACT         = true;
-    final boolean USE_SHARED_CCLASS_TABLE			= false;
     final boolean USE_QTFR_PEEK_NEXT                = true;
 
     final int INIT_MATCH_STACK_SIZE                 = 64;
@@ -66,7 +68,7 @@ public interface Config extends org.jcodings.Config {
 
     final boolean DONT_OPTIMIZE                     = false;
 
-    final boolean USE_STRING_TEMPLATES              = true; // use embeded string templates in Regex object as byte arrays instead of compiling them into int bytecode array
+    final boolean USE_STRING_TEMPLATES              = true; // use embedded string templates in Regex object as byte arrays instead of compiling them into int bytecode array
 
 
     final int MAX_CAPTURE_HISTORY_GROUP             = 31;
@@ -75,9 +77,6 @@ public interface Config extends org.jcodings.Config {
     final int CHECK_STRING_THRESHOLD_LEN            = 7;
     final int CHECK_BUFF_MAX_SIZE                   = 0x4000;
 
-    final boolean NON_UNICODE_SDW                   = true;
-
-
     final PrintStream log = System.out;
     final PrintStream err = System.err;
 
