@@ -21,6 +21,7 @@ package org.joni;
 
 import static org.joni.Option.isAsciiRange;
 import static org.joni.Option.isSingleline;
+import static org.joni.Option.isWordBoundAllRange;
 import static org.joni.ast.QuantifierNode.isRepeatInfinite;
 
 import org.jcodings.Ptr;
@@ -37,9 +38,9 @@ class Lexer extends ScannerSupport {
     protected final Syntax syntax;              // fast access to syntax
     protected final Token token = new Token();  // current token
 
-    protected Lexer(ScanEnvironment env, byte[]bytes, int p, int end) {
-        super(env.enc, bytes, p, end);
-        this.env = env;
+    protected Lexer(Regex regex, Syntax syntax, byte[]bytes, int p, int end, WarnCallback warnings) {
+        super(regex.enc, bytes, p, end);
+        this.env = new ScanEnvironment(regex, syntax, warnings);
         this.syntax = env.syntax;
     }
 
@@ -55,20 +56,20 @@ class Lexer extends ScannerSupport {
             if (synAllow) {
                 return 1; /* "....{" : OK! */
             } else {
-                newSyntaxException(ERR_END_PATTERN_AT_LEFT_BRACE);
+                newSyntaxException(END_PATTERN_AT_LEFT_BRACE);
             }
         }
 
         if (!synAllow) {
             c = peek();
             if (c == ')' || c == '(' || c == '|') {
-                newSyntaxException(ERR_END_PATTERN_AT_LEFT_BRACE);
+                newSyntaxException(END_PATTERN_AT_LEFT_BRACE);
             }
         }
 
         int low = scanUnsignedNumber();
-        if (low < 0) newSyntaxException(ErrorMessages.ERR_TOO_BIG_NUMBER_FOR_REPEAT_RANGE);
-        if (low > Config.MAX_REPEAT_NUM) newSyntaxException(ErrorMessages.ERR_TOO_BIG_NUMBER_FOR_REPEAT_RANGE);
+        if (low < 0) newSyntaxException(ErrorMessages.TOO_BIG_NUMBER_FOR_REPEAT_RANGE);
+        if (low > Config.MAX_REPEAT_NUM) newSyntaxException(ErrorMessages.TOO_BIG_NUMBER_FOR_REPEAT_RANGE);
 
         boolean nonLow = false;
         if (p == _p) { /* can't read low */
@@ -88,8 +89,8 @@ class Lexer extends ScannerSupport {
         if (c == ',') {
             int prev = p; // ??? last
             up = scanUnsignedNumber();
-            if (up < 0) newValueException(ERR_TOO_BIG_NUMBER_FOR_REPEAT_RANGE);
-            if (up > Config.MAX_REPEAT_NUM) newValueException(ERR_TOO_BIG_NUMBER_FOR_REPEAT_RANGE);
+            if (up < 0) newValueException(TOO_BIG_NUMBER_FOR_REPEAT_RANGE);
+            if (up > Config.MAX_REPEAT_NUM) newValueException(TOO_BIG_NUMBER_FOR_REPEAT_RANGE);
 
             if (p == prev) {
                 if (nonLow) return invalidRangeQuantifier(synAllow);
@@ -113,7 +114,7 @@ class Lexer extends ScannerSupport {
         if (c != '}') return invalidRangeQuantifier(synAllow);
 
         if (!isRepeatInfinite(up) && low > up) {
-            newValueException(ERR_UPPER_SMALLER_THAN_LOWER_IN_REPEAT_RANGE);
+            newValueException(UPPER_SMALLER_THAN_LOWER_IN_REPEAT_RANGE);
         }
 
         token.type = TokenType.INTERVAL;
@@ -128,28 +129,26 @@ class Lexer extends ScannerSupport {
             restore();
             return 1;
         } else {
-            newSyntaxException(ERR_INVALID_REPEAT_RANGE_PATTERN);
+            newSyntaxException(INVALID_REPEAT_RANGE_PATTERN);
             return 0; // not reached
         }
     }
 
     /* \M-, \C-, \c, or \... */
-    private int fetchEscapedValue() {
-        if (!left()) newSyntaxException(ERR_END_PATTERN_AT_ESCAPE);
+    private void fetchEscapedValue() {
+        if (!left()) newSyntaxException(END_PATTERN_AT_ESCAPE);
         fetch();
 
         switch(c) {
 
         case 'M':
             if (syntax.op2EscCapitalMBarMeta()) {
-                if (!left()) newSyntaxException(ERR_END_PATTERN_AT_META);
+                if (!left()) newSyntaxException(END_PATTERN_AT_META);
                 fetch();
-                if (c != '-') newSyntaxException(ERR_META_CODE_SYNTAX);
-                if (!left()) newSyntaxException(ERR_END_PATTERN_AT_META);
+                if (c != '-') newSyntaxException(META_CODE_SYNTAX);
+                if (!left()) newSyntaxException(END_PATTERN_AT_META);
                 fetch();
-                if (c == syntax.metaCharTable.esc) {
-                    c = fetchEscapedValue();
-                }
+                if (c == syntax.metaCharTable.esc) fetchEscapedValue();
                 c = ((c & 0xff) | 0x80);
             } else {
                 fetchEscapedValueBackSlash();
@@ -158,9 +157,9 @@ class Lexer extends ScannerSupport {
 
         case 'C':
             if (syntax.op2EscCapitalCBarControl()) {
-                if (!left()) newSyntaxException(ERR_END_PATTERN_AT_CONTROL);
+                if (!left()) newSyntaxException(END_PATTERN_AT_CONTROL);
                 fetch();
-                if (c != '-') newSyntaxException(ERR_CONTROL_CODE_SYNTAX);
+                if (c != '-') newSyntaxException(CONTROL_CODE_SYNTAX);
                 fetchEscapedValueControl();
             } else {
                 fetchEscapedValueBackSlash();
@@ -176,8 +175,6 @@ class Lexer extends ScannerSupport {
         default:
             fetchEscapedValueBackSlash();
         } // switch
-
-        return c; // ???
     }
 
     private void fetchEscapedValueBackSlash() {
@@ -186,19 +183,17 @@ class Lexer extends ScannerSupport {
 
     private void fetchEscapedValueControl() {
         if (!left()) {
-            if (syntax.op2OptionECMAScript()) {
+            if (syntax.op3OptionECMAScript()) {
                 return;
             } else {
-                newSyntaxException(ERR_END_PATTERN_AT_CONTROL);
+                newSyntaxException(END_PATTERN_AT_CONTROL);
             }
         }
         fetch();
         if (c == '?') {
             c = 0177;
         } else {
-            if (c == syntax.metaCharTable.esc) {
-                c = fetchEscapedValue();
-            }
+            if (c == syntax.metaCharTable.esc) fetchEscapedValue();
             c &= 0x9f;
         }
     }
@@ -238,10 +233,10 @@ class Lexer extends ScannerSupport {
 
         String err = null;
         if (!left()) {
-            newValueException(ERR_EMPTY_GROUP_NAME);
+            newValueException(EMPTY_GROUP_NAME);
         } else {
             fetch();
-            if (c == endCode) newValueException(ERR_EMPTY_GROUP_NAME);
+            if (c == endCode) newValueException(EMPTY_GROUP_NAME);
             if (enc.isDigit(c)) {
                 isNum = 1;
             } else if (c == '-') {
@@ -255,7 +250,7 @@ class Lexer extends ScannerSupport {
             nameEnd = p;
             fetch();
             if (c == endCode || c == ')' || c == '+' || c == '-') {
-                if (isNum == 2) err = ERR_INVALID_GROUP_NAME;
+                if (isNum == 2) err = INVALID_GROUP_NAME;
                 break;
             }
 
@@ -263,7 +258,7 @@ class Lexer extends ScannerSupport {
                 if (enc.isDigit(c)) {
                     isNum = 1;
                 } else {
-                    err = ERR_INVALID_GROUP_NAME;
+                    err = INVALID_GROUP_NAME;
                     // isNum = 0;
                 }
             }
@@ -275,10 +270,10 @@ class Lexer extends ScannerSupport {
                 int flag = c == '-' ? -1 : 1;
 
                 fetch();
-                if (!enc.isDigit(c)) newValueException(ERR_INVALID_GROUP_NAME, src, stop);
+                if (!enc.isDigit(c)) newValueException(INVALID_GROUP_NAME, src, stop);
                 unfetch();
                 int level = scanUnsignedNumber();
-                if (level < 0) newValueException(ERR_TOO_BIG_NUMBER);
+                if (level < 0) newValueException(TOO_BIG_NUMBER);
                 rlevel.p = level * flag;
                 existLevel = true;
 
@@ -287,7 +282,7 @@ class Lexer extends ScannerSupport {
             }
 
             if (!isEndCode) {
-                err = ERR_INVALID_GROUP_NAME;
+                err = INVALID_GROUP_NAME;
                 nameEnd = stop;
             }
         }
@@ -299,16 +294,16 @@ class Lexer extends ScannerSupport {
                 int backNum = scanUnsignedNumber();
                 restore();
                 if (backNum < 0) {
-                    newValueException(ERR_TOO_BIG_NUMBER);
+                    newValueException(TOO_BIG_NUMBER);
                 } else if (backNum == 0) {
-                    newValueException(ERR_INVALID_GROUP_NAME, src, stop);
+                    newValueException(INVALID_GROUP_NAME, src, stop);
                 }
                 rbackNum.p = backNum * sign;
             }
             value = nameEnd;
             return existLevel;
         } else {
-            newValueException(ERR_INVALID_GROUP_NAME, src, nameEnd);
+            newValueException(INVALID_GROUP_NAME, src, nameEnd);
             return false; // not reached
         }
     }
@@ -329,15 +324,15 @@ class Lexer extends ScannerSupport {
 
         String err = null;
         if (!left()) {
-            newValueException(ERR_EMPTY_GROUP_NAME);
+            newValueException(EMPTY_GROUP_NAME);
         } else {
             fetch();
-            if (c == endCode) newValueException(ERR_EMPTY_GROUP_NAME);
+            if (c == endCode) newValueException(EMPTY_GROUP_NAME);
             if (enc.isDigit(c)) {
                 if (ref) {
                     isNum = 1;
                 } else {
-                    err = ERR_INVALID_GROUP_NAME;
+                    err = INVALID_GROUP_NAME;
                     // isNum = 0;
                 }
             } else if (c == '-') {
@@ -346,7 +341,7 @@ class Lexer extends ScannerSupport {
                     sign = -1;
                     pnumHead = p;
                 } else {
-                    err = ERR_INVALID_GROUP_NAME;
+                    err = INVALID_GROUP_NAME;
                     // isNum = 0;
                 }
             }
@@ -358,7 +353,7 @@ class Lexer extends ScannerSupport {
                 fetch();
                 if (c == endCode || c == ')') {
                     if (isNum == 2) {
-                        err = ERR_INVALID_GROUP_NAME;
+                        err = INVALID_GROUP_NAME;
                         return fetchNameTeardown(src, endCode, nameEnd, err);
                     }
                     break;
@@ -369,9 +364,9 @@ class Lexer extends ScannerSupport {
                         isNum = 1;
                     } else {
                         if (!enc.isWord(c)) {
-                            err = ERR_INVALID_CHAR_IN_GROUP_NAME;
+                            err = INVALID_CHAR_IN_GROUP_NAME;
                         } else {
-                            err = ERR_INVALID_GROUP_NAME;
+                            err = INVALID_GROUP_NAME;
                         }
                         return fetchNameTeardown(src, endCode, nameEnd, err);
                     }
@@ -379,7 +374,7 @@ class Lexer extends ScannerSupport {
             }
 
             if (c != endCode) {
-                err = ERR_INVALID_GROUP_NAME;
+                err = INVALID_GROUP_NAME;
                 nameEnd = stop;
                 return fetchNameErr(src, nameEnd, err);
             }
@@ -391,9 +386,9 @@ class Lexer extends ScannerSupport {
                 backNum = scanUnsignedNumber();
                 restore();
                 if (backNum < 0) {
-                    newValueException(ERR_TOO_BIG_NUMBER);
+                    newValueException(TOO_BIG_NUMBER);
                 } else if (backNum == 0) {
-                    newValueException(ERR_INVALID_GROUP_NAME, src, nameEnd);
+                    newValueException(INVALID_GROUP_NAME, src, nameEnd);
                 }
                 backNum *= sign;
             }
@@ -434,10 +429,10 @@ class Lexer extends ScannerSupport {
 
         String err = null;
         if (!left()) {
-            newValueException(ERR_EMPTY_GROUP_NAME);
+            newValueException(EMPTY_GROUP_NAME);
         } else {
             fetch();
-            if (c == endCode) newValueException(ERR_EMPTY_GROUP_NAME);
+            if (c == endCode) newValueException(EMPTY_GROUP_NAME);
 
             if (enc.isDigit(c)) {
                 isNum = 1;
@@ -446,7 +441,7 @@ class Lexer extends ScannerSupport {
                 sign = -1;
                 pnumHead = p;
             } else {
-                err = ERR_INVALID_CHAR_IN_GROUP_NAME;
+                err = INVALID_CHAR_IN_GROUP_NAME;
             }
         }
 
@@ -455,11 +450,11 @@ class Lexer extends ScannerSupport {
 
             fetch();
             if (c == endCode || c == ')') break;
-            if (!enc.isDigit(c)) err = ERR_INVALID_CHAR_IN_GROUP_NAME;
+            if (!enc.isDigit(c)) err = INVALID_CHAR_IN_GROUP_NAME;
         }
 
         if (err == null && c != endCode) {
-            err = ERR_INVALID_GROUP_NAME;
+            err = INVALID_GROUP_NAME;
             nameEnd = stop;
         }
 
@@ -469,9 +464,9 @@ class Lexer extends ScannerSupport {
             int backNum = scanUnsignedNumber();
             restore();
             if (backNum < 0) {
-                newValueException(ERR_TOO_BIG_NUMBER);
+                newValueException(TOO_BIG_NUMBER);
             } else if (backNum == 0){
-                newValueException(ERR_INVALID_GROUP_NAME, src, nameEnd);
+                newValueException(INVALID_GROUP_NAME, src, nameEnd);
             }
             backNum *= sign;
 
@@ -547,7 +542,7 @@ class Lexer extends ScannerSupport {
                 }
             }
         } else {
-            syntaxWarn(Warnings.INVALID_UNICODE_PROPERTY, (char)c);
+            syntaxWarn("invalid Unicode Property \\<%n>", (char)c);
         }
     }
 
@@ -557,7 +552,7 @@ class Lexer extends ScannerSupport {
 
         if (peekIs('{') && syntax.opEscXBraceHex8()) {
             inc();
-            int num = scanUnsignedHexadecimalNumber(8);
+            int num = scanUnsignedHexadecimalNumber(0, 8);
             if (num < 0) newValueException(ERR_TOO_BIG_WIDE_CHAR_VALUE);
             if (left()) {
                 int c2 = peek();
@@ -574,8 +569,8 @@ class Lexer extends ScannerSupport {
                 p = last;
             }
         } else if (syntax.opEscXHex2()) {
-            int num = scanUnsignedHexadecimalNumber(2);
-            if (num < 0) newValueException(ERR_TOO_BIG_NUMBER);
+            int num = scanUnsignedHexadecimalNumber(0, 2);
+            if (num < 0) newValueException(TOO_BIG_NUMBER);
             if (p == last) { /* can't read nothing. */
                 num = 0; /* but, it's not error */
             }
@@ -590,8 +585,9 @@ class Lexer extends ScannerSupport {
         int last = p;
 
         if (syntax.op2EscUHex4()) {
-            int num = scanUnsignedHexadecimalNumber(4);
-            if (num < 0) newValueException(ERR_TOO_BIG_NUMBER);
+            int num = scanUnsignedHexadecimalNumber(4, 4);
+            if (num < -1) newValueException(TOO_SHORT_DIGITS);
+            if (num < 0) newValueException(TOO_BIG_NUMBER);
             if (p == last) {  /* can't read nothing. */
                 num = 0; /* but, it's not error */
             }
@@ -606,7 +602,7 @@ class Lexer extends ScannerSupport {
             unfetch();
             int last = p;
             int num = scanUnsignedOctalNumber(3);
-            if (num < 0) newValueException(ERR_TOO_BIG_NUMBER);
+            if (num < 0 || num > 0xff) newValueException(TOO_BIG_NUMBER);
             if (p == last) {  /* can't read nothing. */
                 num = 0; /* but, it's not error */
             }
@@ -665,29 +661,29 @@ class Lexer extends ScannerSupport {
             token.type = TokenType.CC_RANGE;
         } else if (c == syntax.metaCharTable.esc) {
             if (!syntax.backSlashEscapeInCC()) return token.type;
-            if (!left()) newSyntaxException(ERR_END_PATTERN_AT_ESCAPE);
+            if (!left()) newSyntaxException(END_PATTERN_AT_ESCAPE);
             fetch();
             token.escaped = true;
             token.setC(c);
 
             switch (c) {
             case 'w':
-                fetchTokenInCCFor_charType(false, Config.NON_UNICODE_SDW ? CharacterType.W : CharacterType.WORD);
+                fetchTokenInCCFor_charType(false, CharacterType.WORD);
                 break;
             case 'W':
-                fetchTokenInCCFor_charType(true, Config.NON_UNICODE_SDW ? CharacterType.W : CharacterType.WORD);
+                fetchTokenInCCFor_charType(true, CharacterType.WORD);
                 break;
             case 'd':
-                fetchTokenInCCFor_charType(false, Config.NON_UNICODE_SDW ? CharacterType.D : CharacterType.DIGIT);
+                fetchTokenInCCFor_charType(false, CharacterType.DIGIT);
                 break;
             case 'D':
-                fetchTokenInCCFor_charType(true, Config.NON_UNICODE_SDW ? CharacterType.D : CharacterType.DIGIT);
+                fetchTokenInCCFor_charType(true, CharacterType.DIGIT);
                 break;
             case 's':
-                fetchTokenInCCFor_charType(false, Config.NON_UNICODE_SDW ? CharacterType.S : CharacterType.SPACE);
+                fetchTokenInCCFor_charType(false, CharacterType.SPACE);
                 break;
             case 'S':
-                fetchTokenInCCFor_charType(true, Config.NON_UNICODE_SDW ? CharacterType.S : CharacterType.SPACE);
+                fetchTokenInCCFor_charType(true, CharacterType.SPACE);
                 break;
             case 'h':
                 if (syntax.op2EscHXDigit()) fetchTokenInCCFor_charType(false, CharacterType.XDIGIT);
@@ -718,9 +714,9 @@ class Lexer extends ScannerSupport {
 
             default:
                 unfetch();
-                int num = fetchEscapedValue();
-                if (token.getC() != num) {
-                    token.setCode(num);
+                fetchEscapedValue();
+                if (token.getC() != c) {
+                    token.setCode(c);
                     token.type = TokenType.CODE_POINT;
                 }
                 break;
@@ -772,7 +768,7 @@ class Lexer extends ScannerSupport {
         int last = p;
         if (peekIs('{') && syntax.opEscXBraceHex8()) {
             inc();
-            int num = scanUnsignedHexadecimalNumber(8);
+            int num = scanUnsignedHexadecimalNumber(0, 8);
             if (num < 0) newValueException(ERR_TOO_BIG_WIDE_CHAR_VALUE);
             if (left()) {
                 if (enc.isXDigit(peek())) newValueException(ERR_TOO_LONG_WIDE_CHAR_VALUE);
@@ -787,8 +783,8 @@ class Lexer extends ScannerSupport {
                 p = last;
             }
         } else if (syntax.opEscXHex2()) {
-            int num = scanUnsignedHexadecimalNumber(2);
-            if (num < 0) newValueException(ERR_TOO_BIG_NUMBER);
+            int num = scanUnsignedHexadecimalNumber(0, 2);
+            if (num < 0) newValueException(TOO_BIG_NUMBER);
             if (p == last) { /* can't read nothing. */
                 num = 0; /* but, it's not error */
             }
@@ -803,8 +799,9 @@ class Lexer extends ScannerSupport {
         int last = p;
 
         if (syntax.op2EscUHex4()) {
-            int num = scanUnsignedHexadecimalNumber(4);
-            if (num < 0) newValueException(ERR_TOO_BIG_NUMBER);
+            int num = scanUnsignedHexadecimalNumber(4, 4);
+            if (num < -1) newValueException(TOO_SHORT_DIGITS);
+            if (num < 0) newValueException(TOO_BIG_NUMBER);
             if (p == last) { /* can't read nothing. */
                 num = 0; /* but, it's not error */
             }
@@ -821,7 +818,7 @@ class Lexer extends ScannerSupport {
         if (num < 0 || num > Config.MAX_BACKREF_NUM) { // goto skip_backref
         } else if (syntax.opDecimalBackref() && (num <= env.numMem || num <= 9)) { /* This spec. from GNU regex */
             if (syntax.strictCheckBackref()) {
-                if (num > env.numMem || env.memNodes == null || env.memNodes[num] == null) newValueException(ERR_INVALID_BACKREF);
+                if (num > env.numMem || env.memNodes == null || env.memNodes[num] == null) newValueException(INVALID_BACKREF);
             }
             token.type = TokenType.BACKREF;
             token.setBackrefNum(1);
@@ -845,7 +842,7 @@ class Lexer extends ScannerSupport {
         if (syntax.opEscOctal3()) {
             int last = p;
             int num = scanUnsignedOctalNumber(c == '0' ? 2 : 3);
-            if (num < 0) newValueException(ERR_TOO_BIG_NUMBER);
+            if (num < 0 || num > 0xff) newValueException(TOO_BIG_NUMBER);
             if (p == last) { /* can't read nothing. */
                 num = 0; /* but, it's not error */
             }
@@ -865,7 +862,7 @@ class Lexer extends ScannerSupport {
                     fetchNamedBackrefToken();
                 } else {
                     unfetch();
-                    syntaxWarn(Warnings.INVALID_BACKREFERENCE);
+                    syntaxWarn("invalid back reference");
                 }
             }
         }
@@ -912,14 +909,14 @@ class Lexer extends ScannerSupport {
                     token.setCallGNum(gNum);
                     token.setCallRel(rel);
                 } else {
-                    syntaxWarn(Warnings.INVALID_SUBEXP_CALL);
+                    syntaxWarn("invalid subexp call");
                     unfetch();
                 }
             }
         }
     }
 
-    private void fetchNamedBackrefToken() {
+    protected void fetchNamedBackrefToken() {
         int last = p;
         int backNum;
         if (Config.USE_BACKREF_WITH_LEVEL) {
@@ -936,11 +933,11 @@ class Lexer extends ScannerSupport {
         if (backNum != 0) {
             if (backNum < 0) {
                 backNum = backrefRelToAbs(backNum);
-                if (backNum <= 0) newValueException(ERR_INVALID_BACKREF);
+                if (backNum <= 0) newValueException(INVALID_BACKREF);
             }
 
             if (syntax.strictCheckBackref() && (backNum > env.numMem || env.memNodes == null)) {
-                newValueException(ERR_INVALID_BACKREF);
+                newValueException(INVALID_BACKREF);
             }
             token.type = TokenType.BACKREF;
             token.setBackrefByName(false);
@@ -948,18 +945,18 @@ class Lexer extends ScannerSupport {
             token.setBackrefRef1(backNum);
         } else {
             NameEntry e = env.reg.nameToGroupNumbers(bytes, last, nameEnd);
-            if (e == null) newValueException(ERR_UNDEFINED_NAME_REFERENCE, last, nameEnd);
+            if (e == null) newValueException(UNDEFINED_NAME_REFERENCE, last, nameEnd);
 
             if (syntax.strictCheckBackref()) {
                 if (e.backNum == 1) {
                     if (e.backRef1 > env.numMem ||
                         env.memNodes == null ||
-                        env.memNodes[e.backRef1] == null) newValueException(ERR_INVALID_BACKREF);
+                        env.memNodes[e.backRef1] == null) newValueException(INVALID_BACKREF);
                 } else {
                     for (int i=0; i<e.backNum; i++) {
                         if (e.backRefs[i] > env.numMem ||
                             env.memNodes == null ||
-                            env.memNodes[e.backRefs[i]] == null) newValueException(ERR_INVALID_BACKREF);
+                            env.memNodes[e.backRefs[i]] == null) newValueException(INVALID_BACKREF);
                     }
                 }
             }
@@ -992,7 +989,7 @@ class Lexer extends ScannerSupport {
                 }
             }
         } else {
-            syntaxWarn(Warnings.INVALID_UNICODE_PROPERTY, (char)c);
+            syntaxWarn("invalid Unicode Property \\<%n>", (char)c);
         }
     }
 
@@ -1027,7 +1024,7 @@ class Lexer extends ScannerSupport {
             fetch();
 
             if (c == syntax.metaCharTable.esc && !syntax.op2IneffectiveEscape()) { // IS_MC_ESC_CODE(code, syn)
-                if (!left()) newSyntaxException(ERR_END_PATTERN_AT_ESCAPE);
+                if (!left()) newSyntaxException(END_PATTERN_AT_ESCAPE);
 
                 token.backP = p;
                 fetch();
@@ -1058,21 +1055,21 @@ class Lexer extends ScannerSupport {
                     if (syntax.opEscLParenSubexp()) token.type = TokenType.SUBEXP_CLOSE;
                     break;
                 case 'w':
-                    if (syntax.opEscWWord()) fetchTokenInCCFor_charType(false, Config.NON_UNICODE_SDW ? CharacterType.W : CharacterType.WORD);
+                    if (syntax.opEscWWord()) fetchTokenInCCFor_charType(false, CharacterType.WORD);
                     break;
                 case 'W':
-                    if (syntax.opEscWWord()) fetchTokenInCCFor_charType(true, Config.NON_UNICODE_SDW ? CharacterType.W : CharacterType.WORD);
+                    if (syntax.opEscWWord()) fetchTokenInCCFor_charType(true, CharacterType.WORD);
                     break;
                 case 'b':
                     if (syntax.opEscBWordBound()) {
                         fetchTokenFor_anchor(AnchorType.WORD_BOUND);
-                        token.setAnchorASCIIRange(isAsciiRange(env.option));
+                        token.setAnchorASCIIRange(isAsciiRange(env.option) && !isWordBoundAllRange(env.option));
                     }
                     break;
                 case 'B':
                     if (syntax.opEscBWordBound()) {
                         fetchTokenFor_anchor(AnchorType.NOT_WORD_BOUND);
-                        token.setAnchorASCIIRange(isAsciiRange(env.option));
+                        token.setAnchorASCIIRange(isAsciiRange(env.option) && !isWordBoundAllRange(env.option));
                     }
                     break;
                 case '<':
@@ -1088,16 +1085,16 @@ class Lexer extends ScannerSupport {
                     }
                     break;
                 case 's':
-                    if (syntax.opEscSWhiteSpace()) fetchTokenInCCFor_charType(false, Config.NON_UNICODE_SDW ? CharacterType.S : CharacterType.SPACE);
+                    if (syntax.opEscSWhiteSpace()) fetchTokenInCCFor_charType(false, CharacterType.SPACE);
                     break;
                 case 'S':
-                    if (syntax.opEscSWhiteSpace()) fetchTokenInCCFor_charType(true, Config.NON_UNICODE_SDW ? CharacterType.S : CharacterType.SPACE);
+                    if (syntax.opEscSWhiteSpace()) fetchTokenInCCFor_charType(true, CharacterType.SPACE);
                     break;
                 case 'd':
-                    if (syntax.opEscDDigit()) fetchTokenInCCFor_charType(false, Config.NON_UNICODE_SDW ? CharacterType.D : CharacterType.DIGIT);
+                    if (syntax.opEscDDigit()) fetchTokenInCCFor_charType(false, CharacterType.DIGIT);
                     break;
                 case 'D':
-                    if (syntax.opEscDDigit()) fetchTokenInCCFor_charType(true, Config.NON_UNICODE_SDW ? CharacterType.D : CharacterType.DIGIT);
+                    if (syntax.opEscDDigit()) fetchTokenInCCFor_charType(true, CharacterType.DIGIT);
                     break;
                 case 'h':
                     if (syntax.op2EscHXDigit()) fetchTokenInCCFor_charType(false, CharacterType.XDIGIT);
@@ -1166,19 +1163,15 @@ class Lexer extends ScannerSupport {
                     if (syntax.op2EscCapitalKKeep()) token.type = TokenType.KEEP;
                 default:
                     unfetch();
-                    int num = fetchEscapedValue();
-
-                    /* set_raw: */
-                    if (token.getC() != num) {
+                    fetchEscapedValue();
+                    if (token.getC() != c) { /* set_raw: */
                         token.type = TokenType.CODE_POINT;
-                        token.setCode(num);
+                        token.setCode(c);
                     } else { /* string */
                         p = token.backP + enc.length(bytes, token.backP, stop);
                     }
                     break;
-
                 } // switch (c)
-
             } else {
                 token.setC(c);
                 token.escaped = false;
@@ -1215,7 +1208,7 @@ class Lexer extends ScannerSupport {
                             if (peekIs('#')) {
                                 fetch();
                                 while (true) {
-                                    if (!left()) newSyntaxException(ERR_END_PATTERN_IN_GROUP);
+                                    if (!left()) newSyntaxException(END_PATTERN_IN_GROUP);
                                     fetch();
                                     if (c == syntax.metaCharTable.esc) {
                                         if (left()) fetch();
@@ -1240,7 +1233,7 @@ class Lexer extends ScannerSupport {
                         if (syntax.opLineAnchor()) fetchTokenFor_anchor(isSingleline(env.option) ? AnchorType.SEMI_END_BUF : AnchorType.END_LINE);
                         break;
                     case '[':
-                        if (syntax.opBracketCC()) token.type = TokenType.CC_CC_OPEN;
+                        if (syntax.opBracketCC()) token.type = TokenType.CC_OPEN;
                         break;
                     case ']':
                         //if (*src > env->pattern)   /* /].../ is allowed. */
@@ -1314,7 +1307,7 @@ class Lexer extends ScannerSupport {
                 throw new CharacterPropertyException(ERR_INVALID_CHAR_PROPERTY_NAME, bytes, _p, last);
             }
         }
-        newInternalException(ERR_PARSER_BUG);
+        newInternalException(PARSER_BUG);
         return 0; // not reached
     }
 
@@ -1324,7 +1317,7 @@ class Lexer extends ScannerSupport {
 
     protected final void syntaxWarn(String message) {
         if (Config.USE_WARN) {
-            env.reg.warnings.warn(message + ": /" + new String(bytes, getBegin(), getEnd()) + "/");
+            env.warnings.warn(message + ": /" + new String(bytes, getBegin(), getEnd()) + "/");
         }
     }
 }
