@@ -24,12 +24,14 @@ import static org.joni.Option.isFindLongest;
 
 import org.jcodings.Encoding;
 import org.jcodings.IntHolder;
+import org.jcodings.constants.CharacterType;
+import org.jcodings.specific.ASCIIEncoding;
 import org.joni.constants.AnchorType;
 
 public abstract class Matcher extends IntHolder {
     public static final int FAILED = -1;
     public static final int INTERRUPTED = -2;
-    
+
     protected final Regex regex;
     protected final Encoding enc;
 
@@ -46,14 +48,9 @@ public abstract class Matcher extends IntHolder {
     protected int msaBegin;
     protected int msaEnd;
 
-    public Matcher(Regex regex, byte[]bytes) {
-        this(regex, bytes, 0, bytes.length);
-    }
-
-    public Matcher(Regex regex, byte[]bytes, int p, int end) {
+    Matcher(Regex regex, byte[]bytes, int p, int end) {
         this.regex = regex;
         this.enc = regex.enc;
-
         this.bytes = bytes;
         this.str = p;
         this.end = end;
@@ -88,7 +85,7 @@ public abstract class Matcher extends IntHolder {
         msaStart = start;
         if (Config.USE_FIND_LONGEST_SEARCH_ALL_OF_RANGE) msaBestLen = -1;
     }
-    
+
     public final int match(int at, int range, int option) {
         try {
             return matchInterruptible(at, range, option);
@@ -96,11 +93,11 @@ public abstract class Matcher extends IntHolder {
             return INTERRUPTED;
         }
     }
-    
+
     public final int matchInterruptible(int at, int range, int option) throws InterruptedException {
         msaInit(option, at);
 
-        if (Config.USE_COMBINATION_EXPLOSION_CHECK) {
+        if (Config.USE_CEC) {
             int offset = at = str;
             stateCheckBuffInit(end - str, offset, regex.numCombExpCheck); // move it to construction?
         } // USE_COMBINATION_EXPLOSION_CHECK
@@ -115,17 +112,11 @@ public abstract class Matcher extends IntHolder {
     }
 
     int low, high; // these are the return values
-    private boolean forwardSearchRange(byte[]bytes, int str, int end, int s, int range, IntHolder lowPrev) {
+    private final boolean forwardSearchRange(byte[]bytes, int str, int end, int s, int range, IntHolder lowPrev) {
         int pprev = -1;
         int p = s;
 
-        if (Config.DEBUG_SEARCH) {
-            Config.log.println("forward_search_range: "+
-                                "str: " + str +
-                                ", end: " + end +
-                                ", s: " + s +
-                                ", range: " + range);
-        }
+        if (Config.DEBUG_SEARCH) debugForwardSearchRange(str, end, s, range);
 
         if (regex.dMin > 0) {
             if (enc.isSingleByte()) {
@@ -210,24 +201,15 @@ public abstract class Matcher extends IntHolder {
                 }
                 /* no needs to adjust *high, *high is used as range check only */
                 high = p - regex.dMin;
-
-                if (Config.DEBUG_SEARCH) {
-                    Config.log.println("forward_search_range success: "+
-                                        "low: " + (low - str) +
-                                        ", high: " + (high - str) +
-                                        ", dmin: " + regex.dMin +
-                                        ", dmax: " + regex.dMax);
-                }
-
+                if (Config.DEBUG_SEARCH) debugForwardSearchRangeSuccess(str, low, high);
                 return true;    /* success */
             }
-
             return false;   /* fail */
         } //while
     }
 
     // low, high
-    private boolean backwardSearchRange(byte[]bytes, int str, int end, int s, int range, int adjrange) {
+    private final boolean backwardSearchRange(byte[]bytes, int str, int end, int s, int range, int adjrange) {
         range += regex.dMin;
         int p = s;
 
@@ -273,12 +255,7 @@ public abstract class Matcher extends IntHolder {
                     high = enc.rightAdjustCharHead(bytes, adjrange, high, end);
                 }
 
-                if (Config.DEBUG_SEARCH) {
-                    Config.log.println("backward_search_range: "+
-                                        "low: " + (low - str) +
-                                        ", high: " + (high - str));
-                }
-
+                if (Config.DEBUG_SEARCH) debugBackwardSearchRange(str, low, high);
                 return true;
             }
 
@@ -312,7 +289,7 @@ public abstract class Matcher extends IntHolder {
         }
         return false;
     }
-    
+
     public final int search(int start, int range, int option) {
         try {
             return searchInterruptible(start, range, option);
@@ -326,13 +303,7 @@ public abstract class Matcher extends IntHolder {
         int origStart = start;
         int origRange = range;
 
-        if (Config.DEBUG_SEARCH) {
-            Config.log.println("onig_search (entry point): "+
-            		"str: " + str +
-                    ", end: " + (end - str) +
-                    ", start: " + (start - str) +
-                    ", range " + (range - str));
-        }
+        if (Config.DEBUG_SEARCH) debugSearch(str, end, start, range);
 
         if (start > end || start < str) return FAILED;
 
@@ -396,16 +367,14 @@ public abstract class Matcher extends IntHolder {
 
         } else if (str == end) { /* empty string */
             // empty address ?
-            if (Config.DEBUG_SEARCH) {
-                Config.log.println("onig_search: empty string.");
-            }
+            if (Config.DEBUG_SEARCH) Config.log.println("onig_search: empty string.");
 
             if (regex.thresholdLength == 0) {
                 s = start = str;
                 prev = -1;
                 msaInit(option, start);
 
-                if (Config.USE_COMBINATION_EXPLOSION_CHECK) stateCheckBuffClear();
+                if (Config.USE_CEC) stateCheckBuffClear();
 
                 if (matchCheck(end, s, prev)) return match(s);
                 return mismatch();
@@ -413,15 +382,10 @@ public abstract class Matcher extends IntHolder {
             return FAILED; // goto mismatch_no_msa;
         }
 
-        if (Config.DEBUG_SEARCH) {
-            Config.log.println("onig_search(apply anchor): " +
-                                "end: " + (end - str) +
-                                ", start " + (start - str) +
-                                ", range " + (range - str));
-        }
+        if (Config.DEBUG_SEARCH) debugSearch(str, end, start, range);
 
         msaInit(option, origStart);
-        if (Config.USE_COMBINATION_EXPLOSION_CHECK) {
+        if (Config.USE_CEC) {
             int offset = Math.min(start, range) - str;
             stateCheckBuffInit(end - str, offset, regex.numCombExpCheck);
         }
@@ -469,10 +433,16 @@ public abstract class Matcher extends IntHolder {
                             if (matchCheck(origRange, s, prev)) return match(s);
                             prev = s;
                             s += enc.length(bytes, s, end);
+
+                            if ((regex.anchor & (AnchorType.LOOK_BEHIND | AnchorType.PREC_READ_NOT)) == 0) {
+                                while (!enc.isNewLine(bytes, prev, end) && s < range) {
+                                    prev = s;
+                                    s += enc.length(bytes, s, end);
+                                }
+                            }
                         } while (s < range);
                         return mismatch();
                     }
-
                 }
             }
 
@@ -542,7 +512,7 @@ public abstract class Matcher extends IntHolder {
         return mismatch();
     }
 
-    private boolean endBuf(int start, int range, int minSemiEnd, int maxSemiEnd) {
+    private final boolean endBuf(int start, int range, int minSemiEnd, int maxSemiEnd) {
         if ((maxSemiEnd - str) < regex.anchorDmin) return true; // mismatch_no_msa;
 
         if (range > start) {
@@ -571,11 +541,11 @@ public abstract class Matcher extends IntHolder {
         return false;
     }
 
-    private int match(int s) {
+    private final int match(int s) {
         return s - str; // sstart ???
     }
 
-    private int mismatch() {
+    private final int mismatch() {
         if (Config.USE_FIND_LONGEST_SEARCH_ALL_OF_RANGE) {
             if (msaBestLen >= 0) {
                 int s = msaBestS;
@@ -588,8 +558,49 @@ public abstract class Matcher extends IntHolder {
 
     private byte[]icbuf;
 
-    final byte[]icbuf() {
+    protected final byte[]icbuf() {
         return icbuf == null ? icbuf = new byte[Config.ENC_MBC_CASE_FOLD_MAXLEN] : icbuf;
     }
 
+    static boolean isMbcAsciiWord(Encoding enc, byte[]bytes, int p, int end) { // ONIGENC_IS_MBC_ASCII_WORD
+        return ASCIIEncoding.INSTANCE.isCodeCType(enc.mbcToCode(bytes, p, end), CharacterType.WORD);
+    }
+
+    private final void debugForwardSearchRange(int str, int end, int s, int range) {
+        if (Config.DEBUG_SEARCH) {
+            Config.log.println("forward_search_range: " +
+                "str: " + str +
+                ", end: " + end +
+                ", s: " + s +
+                ", range: " + range);
+        }
+    }
+
+    private final void debugForwardSearchRangeSuccess(int str, int low, int high) {
+        if (Config.DEBUG_SEARCH) {
+            Config.log.println("forward_search_range success: " +
+                "low: " + (low - str) +
+                ", high: " + (high - str) +
+                ", dmin: " + regex.dMin +
+                ", dmax: " + regex.dMax);
+        }
+    }
+
+    private final void debugSearch(int str, int end, int start, int range) {
+        if (Config.DEBUG_SEARCH) {
+            Config.log.println("onig_search (entry point): " +
+                "str: " + str +
+                ", end: " + (end - str) +
+                ", start: " + (start - str) +
+                ", range " + (range - str));
+        }
+    }
+
+    private final void debugBackwardSearchRange(int str, int low, int high) {
+        if (Config.DEBUG_SEARCH) {
+            Config.log.println("backward_search_range: "+
+                "low: " + (low - str) +
+                ", high: " + (high - str));
+        }
+    }
 }
