@@ -20,6 +20,7 @@
 package org.joni;
 
 import static org.joni.BitStatus.bsAt;
+import static org.joni.Config.USE_CEC;
 
 import java.lang.ref.WeakReference;
 import java.util.Arrays;
@@ -32,13 +33,10 @@ abstract class StackMachine extends Matcher implements StackType {
 
     protected StackEntry[]stack;
     protected int stk;  // stkEnd
-
     protected final int[]repeatStk;
     protected final int memStartStk, memEndStk;
-
-    // CEC
-    protected byte[] stateCheckBuff; // move to int[] ?
-    int stateCheckBuffSize;
+    protected byte[] stateCheckBuff; // CEC, move to int[] ?
+    protected int stateCheckBuffSize;
 
     protected StackMachine(Regex regex, byte[]bytes, int p , int end) {
         super(regex, bytes, p, end);
@@ -58,9 +56,18 @@ abstract class StackMachine extends Matcher implements StackType {
         repeatStk = n > 0 ? new int[n] : null;
     }
 
+    protected final void stackInit() {
+        if (stack != null) pushEnsured(ALT, regex.codeLength - 1); /* bottom stack */
+        if (repeatStk != null) {
+            for (int i = (Config.USE_SUBEXP_CALL ? 0 : 1); i <= regex.numMem; i++) {
+                repeatStk[i + memStartStk] = repeatStk[i + memEndStk] = INVALID_INDEX;
+            }
+        }
+    }
+
     private static StackEntry[] allocateStack() {
         StackEntry[]stack = new StackEntry[Config.INIT_MATCH_STACK_SIZE];
-        stack[0] = new StackEntry();
+        stack[0] = USE_CEC ? new SCStackEntry() : new StackEntry();
         return stack;
     }
 
@@ -88,23 +95,14 @@ abstract class StackMachine extends Matcher implements StackType {
         return stack;
     }
 
-    protected final void init() {
-        if (stack != null) pushEnsured(ALT, regex.codeLength - 1); /* bottom stack */
-        if (repeatStk != null) {
-            for (int i = (Config.USE_SUBEXP_CALL ? 0 : 1); i <= regex.numMem; i++) {
-                repeatStk[i + memStartStk] = repeatStk[i + memEndStk] = INVALID_INDEX;
-            }
-        }
-    }
-
-    protected final StackEntry ensure1() {
+    private final StackEntry ensure1() {
         if (stk >= stack.length) doubleStack();
         StackEntry e = stack[stk];
-        if (e == null) stack[stk] = e = new StackEntry();
+        if (e == null) stack[stk] = e = USE_CEC ? new SCStackEntry() : new StackEntry();
         return e;
     }
 
-    protected final void pushType(int type) {
+    private final void pushType(int type) {
         ensure1().type = type;
         stk++;
     }
@@ -128,7 +126,7 @@ abstract class StackMachine extends Matcher implements StackType {
     // ELSE_IF_STATE_CHECK_MARK
     private void stateCheckMark() {
         StackEntry e = stack[stk];
-        int x = stateCheckPos(e.getStatePStr(), e.getStateCheck());
+        int x = stateCheckPos(e.getStatePStr(), ((SCStackEntry)e).getStateCheck());
         stateCheckBuff[x / 8] |= (1 << (x % 8));
     }
 
@@ -163,31 +161,33 @@ abstract class StackMachine extends Matcher implements StackType {
         stateCheckBuffSize = 0;
     }
 
-    private void push(int type, int pat, int s, int prev) {
+    private void push(int type, int pat, int s, int prev, int pkeep) {
         StackEntry e = ensure1();
         e.type = type;
         e.setStatePCode(pat);
         e.setStatePStr(s);
         e.setStatePStrPrev(prev);
-        if (Config.USE_COMBINATION_EXPLOSION_CHECK) e.setStateCheck(0);
+        if (USE_CEC) ((SCStackEntry)e).setStateCheck(0);
+        e.setPKeep(pkeep);
         stk++;
     }
 
-    protected final void pushEnsured(int type, int pat) {
+    private final void pushEnsured(int type, int pat) {
         StackEntry e = stack[stk];
         e.type = type;
         e.setStatePCode(pat);
-        if (Config.USE_COMBINATION_EXPLOSION_CHECK) e.setStateCheck(0);
+        if (USE_CEC) ((SCStackEntry)e).setStateCheck(0);
         stk++;
     }
 
-    protected final void pushAltWithStateCheck(int pat, int s, int sprev, int snum) {
+    protected final void pushAltWithStateCheck(int pat, int s, int sprev, int snum, int pkeep) {
         StackEntry e = ensure1();
         e.type = ALT;
         e.setStatePCode(pat);
         e.setStatePStr(s);
         e.setStatePStrPrev(sprev);
-        if (Config.USE_COMBINATION_EXPLOSION_CHECK) e.setStateCheck(stateCheckBuff != null ? snum : 0);
+        if (USE_CEC) ((SCStackEntry)e).setStateCheck(stateCheckBuff != null ? snum : 0);
+        e.setPKeep(pkeep);
         stk++;
     }
 
@@ -196,29 +196,29 @@ abstract class StackMachine extends Matcher implements StackType {
             StackEntry e = ensure1();
             e.type = STATE_CHECK_MARK;
             e.setStatePStr(s);
-            e.setStateCheck(snum);
+            ((SCStackEntry)e).setStateCheck(snum);
             stk++;
         }
     }
 
-    protected final void pushAlt(int pat, int s, int prev) {
-        push(ALT, pat, s, prev);
+    protected final void pushAlt(int pat, int s, int prev, int pkeep) {
+        push(ALT, pat, s, prev, pkeep);
     }
 
-    protected final void pushPos(int s, int prev) {
-        push(POS, -1 /*NULL_UCHARP*/, s, prev);
+    protected final void pushPos(int s, int prev, int pkeep) {
+        push(POS, -1 /*NULL_UCHARP*/, s, prev, pkeep);
     }
 
-    protected final void pushPosNot(int pat, int s, int prev) {
-        push(POS_NOT, pat, s, prev);
+    protected final void pushPosNot(int pat, int s, int prev, int pkeep) {
+        push(POS_NOT, pat, s, prev, pkeep);
     }
 
     protected final void pushStopBT() {
         pushType(STOP_BT);
     }
 
-    protected final void pushLookBehindNot(int pat, int s, int sprev) {
-        push(LOOK_BEHIND_NOT, pat, s, sprev);
+    protected final void pushLookBehindNot(int pat, int s, int sprev, int pkeep) {
+        push(LOOK_BEHIND_NOT, pat, s, sprev, pkeep);
     }
 
     protected final void pushRepeat(int id, int pat) {
@@ -312,8 +312,19 @@ abstract class StackMachine extends Matcher implements StackType {
         stk++;
     }
 
-    // stack debug routines here
-    // ...
+    protected final void pushAbsent() {
+        StackEntry e = ensure1();
+        e.type = ABSENT;
+        stk++;
+    }
+
+    protected final void pushAbsentPos(int start, int end) {
+        StackEntry e = ensure1();
+        e.type = ABSENT_POS;
+        e.setAbsentStr(start);
+        e.setAbsentEndStr(end);
+        stk++;
+    }
 
     protected final void popOne() {
         stk--;
@@ -333,10 +344,9 @@ abstract class StackMachine extends Matcher implements StackType {
     private StackEntry popFree() {
         while (true) {
             StackEntry e = stack[--stk];
-
             if ((e.type & MASK_POP_USED) != 0) {
                 return e;
-            } else if (Config.USE_COMBINATION_EXPLOSION_CHECK) {
+            } else if (USE_CEC) {
                 if (e.type == STATE_CHECK_MARK) stateCheckMark();
             }
         }
@@ -345,83 +355,56 @@ abstract class StackMachine extends Matcher implements StackType {
     private StackEntry popMemStart() {
         while (true) {
             StackEntry e = stack[--stk];
-
             if ((e.type & MASK_POP_USED) != 0) {
                 return e;
             } else if (e.type == MEM_START) {
                 repeatStk[memStartStk + e.getMemNum()] = e.getMemStart();
                 repeatStk[memEndStk + e.getMemNum()] = e.getMemEnd();
-            } else if (Config.USE_COMBINATION_EXPLOSION_CHECK) {
+            } else if (USE_CEC) {
                 if (e.type == STATE_CHECK_MARK) stateCheckMark();
             }
         }
     }
 
+    private void popRewrite(StackEntry e) {
+        if (e.type == MEM_START) {
+            repeatStk[memStartStk + e.getMemNum()] = e.getMemStart();
+            repeatStk[memEndStk + e.getMemNum()] = e.getMemEnd();
+        } else if (e.type == REPEAT_INC) {
+            stack[e.getSi()].decreaseRepeatCount();
+        } else if (e.type == MEM_END) {
+            repeatStk[memStartStk + e.getMemNum()] = e.getMemStart();
+            repeatStk[memEndStk + e.getMemNum()] = e.getMemEnd();
+        } else if (USE_CEC) {
+            if (e.type == STATE_CHECK_MARK) stateCheckMark();
+        }
+    }
+
     private StackEntry popDefault() {
         while (true) {
             StackEntry e = stack[--stk];
-
-            if ((e.type & MASK_POP_USED) != 0) {
-                return e;
-            } else if (e.type == MEM_START) {
-                repeatStk[memStartStk + e.getMemNum()] = e.getMemStart();
-                repeatStk[memEndStk + e.getMemNum()] = e.getMemEnd();
-            } else if (e.type == REPEAT_INC) {
-                //int si = stack[stk + IREPEAT_INC_SI];
-                //stack[si + IREPEAT_COUNT]--;
-                stack[e.getSi()].decreaseRepeatCount();
-            } else if (e.type == MEM_END) {
-                repeatStk[memStartStk + e.getMemNum()] = e.getMemStart();
-                repeatStk[memEndStk + e.getMemNum()] = e.getMemEnd();
-            } else if (Config.USE_COMBINATION_EXPLOSION_CHECK) {
-                if (e.type == STATE_CHECK_MARK) stateCheckMark();
-            }
+            if ((e.type & MASK_POP_USED) != 0) return e; else popRewrite(e);
         }
     }
 
     protected final void popTilPosNot() {
         while (true) {
-            stk--;
-            StackEntry e = stack[stk];
-
-            if (e.type == POS_NOT) {
-                break;
-            } else if (e.type == MEM_START) {
-                repeatStk[memStartStk + e.getMemNum()] = e.getMemStart();
-                repeatStk[memEndStk + e.getMemNum()] = e.getMemEnd();
-            } else if (e.type == REPEAT_INC) {
-                //int si = stack[stk + IREPEAT_INC_SI];
-                //stack[si + IREPEAT_COUNT]--;
-                stack[e.getSi()].decreaseRepeatCount();
-            } else if (e.type == MEM_END){
-                repeatStk[memStartStk + e.getMemNum()] = e.getMemStart();
-                repeatStk[memEndStk + e.getMemNum()] = e.getMemEnd();
-            } else if (Config.USE_COMBINATION_EXPLOSION_CHECK) {
-                if (e.type == STATE_CHECK_MARK) stateCheckMark();
-            }
+            StackEntry e = stack[--stk];
+            if (e.type == POS_NOT) break; else popRewrite(e);
         }
     }
 
     protected final void popTilLookBehindNot() {
         while (true) {
-            stk--;
-            StackEntry e = stack[stk];
+            StackEntry e = stack[--stk];
+            if (e.type == LOOK_BEHIND_NOT) break; else popRewrite(e);
+        }
+    }
 
-            if (e.type == LOOK_BEHIND_NOT) {
-                break;
-            } else if (e.type == MEM_START) {
-                repeatStk[memStartStk + e.getMemNum()] = e.getMemStart();
-                repeatStk[memEndStk + e.getMemNum()] = e.getMemEnd();
-            } else if (e.type == REPEAT_INC) {
-                //int si = stack[stk + IREPEAT_INC_SI];
-                //stack[si + IREPEAT_COUNT]--;
-                stack[e.getSi()].decreaseRepeatCount();
-            } else if (e.type == MEM_END) {
-                repeatStk[memStartStk + e.getMemNum()] = e.getMemStart();
-                repeatStk[memEndStk + e.getMemNum()] = e.getMemEnd();
-            } else if (Config.USE_COMBINATION_EXPLOSION_CHECK) {
-                if (e.type == STATE_CHECK_MARK) stateCheckMark();
-            }
+    protected final void popTilAbsent() {
+        while (true) {
+            StackEntry e = stack[--stk];
+            if (e.type == ABSENT) break; else popRewrite(e);
         }
     }
 
