@@ -23,16 +23,20 @@ import static org.joni.constants.MetaChar.INEFFECTIVE_META_CHAR;
 
 import org.joni.constants.SyntaxProperties;
 
-public final class Syntax implements SyntaxProperties{
+public final class Syntax implements SyntaxProperties {
+    public final String name;
     private final int op;
     private final int op2;
+    private final int op3;
     private final int behavior;
     public final int options;
     public final MetaCharTable metaCharTable;
 
-    public Syntax(int op, int op2, int behavior, int options, MetaCharTable metaCharTable) {
+    public Syntax(String name, int op, int op2, int op3, int behavior, int options, MetaCharTable metaCharTable) {
+        this.name = name;
         this.op = op;
         this.op2 = op2;
+        this.op3 = op3;
         this.behavior = behavior;
         this.options = options;
         this.metaCharTable = metaCharTable;
@@ -301,6 +305,10 @@ public final class Syntax implements SyntaxProperties{
         return isOp2(OP2_ESC_CAPITAL_K_KEEP);
     }
 
+    public boolean op2QMarkTildeAbsent() {
+        return isOp2(OP2_QMARK_TILDE_ABSENT);
+    }
+
     public boolean op2EscGBraceBackref() {
         return isOp2(OP2_ESC_G_BRACE_BACKREF);
     }
@@ -321,12 +329,16 @@ public final class Syntax implements SyntaxProperties{
         return isOp2(OP2_QMARK_CAPITAL_P_NAMED_GROUP);
     }
 
-    public boolean op2OptionJava() {
-        return isOp2(OP2_OPTION_JAVA);
+    protected boolean isOp3(int opm) {
+        return (op3 & opm) != 0;
     }
 
-    public boolean op2OptionECMAScript() {
-        return isOp2(OP2_OPTION_ECMASCRIPT);
+    public boolean op3OptionJava() {
+        return isOp3(OP3_OPTION_JAVA);
+    }
+
+    public boolean op3OptionECMAScript() {
+        return isOp3(OP3_OPTION_ECMASCRIPT);
     }
 
 
@@ -399,11 +411,16 @@ public final class Syntax implements SyntaxProperties{
         return isBehavior(WARN_CC_OP_NOT_ESCAPED);
     }
 
+    public boolean warnCCDup() {
+        return isBehavior(WARN_CC_DUP);
+    }
+
     public boolean warnReduntantNestedRepeat() {
         return isBehavior(WARN_REDUNDANT_NESTED_REPEAT);
     }
 
     public static final Syntax RUBY = new Syntax(
+        "RUBY",
         (( GNU_REGEX_OP | OP_QMARK_NON_GREEDY |
         OP_ESC_OCTAL3 | OP_ESC_X_HEX2 |
         OP_ESC_X_BRACE_HEX8 | OP_ESC_CONTROL_CHARS |
@@ -422,9 +439,13 @@ public final class Syntax implements SyntaxProperties{
         OP2_ESC_H_XDIGIT |
         OP2_ESC_CAPITAL_X_EXTENDED_GRAPHEME_CLUSTER |
         OP2_QMARK_LPAREN_CONDITION |
-        OP2_ESC_CAPITAL_R_LINEBREAK // | OP2_ESC_CAPITAL_K_KEEP
+        OP2_ESC_CAPITAL_R_LINEBREAK |
+        OP2_ESC_CAPITAL_K_KEEP |
+        OP2_QMARK_TILDE_ABSENT
         ),
 
+        0,
+
         ( GNU_REGEX_BV |
         ALLOW_INTERVAL_LOW_ABBREV |
         DIFFERENT_LEN_ALT_LOOK_BEHIND |
@@ -432,9 +453,10 @@ public final class Syntax implements SyntaxProperties{
         ALLOW_MULTIPLEX_DEFINITION_NAME |
         FIXED_INTERVAL_IS_GREEDY_ONLY |
         WARN_CC_OP_NOT_ESCAPED |
+        WARN_CC_DUP |
         WARN_REDUNDANT_NESTED_REPEAT ),
 
-        Option.NONE,
+        (Option.ASCII_RANGE | Option.POSIX_BRACKET_ALL_RANGE | Option.WORD_BOUND_ALL_RANGE),
 
         new MetaCharTable(
             '\\',                           /* esc */
@@ -448,13 +470,18 @@ public final class Syntax implements SyntaxProperties{
 
     public static final Syntax DEFAULT = RUBY;
 
+    public static final Syntax TEST = new Syntax("TEST", RUBY.op, RUBY.op2 | OP2_ESC_U_HEX4, RUBY.op3, RUBY.behavior, RUBY.options & ~ Option.ASCII_RANGE, RUBY.metaCharTable);
+
     public static final Syntax ASIS = new Syntax(
+        "ASIS",
         0,
 
         OP2_INEFFECTIVE_ESCAPE,
 
         0,
 
+        0,
+
         Option.NONE,
 
         new MetaCharTable(
@@ -468,6 +495,7 @@ public final class Syntax implements SyntaxProperties{
     );
 
     public static final Syntax PosixBasic = new Syntax(
+        "PosixBasic",
         (POSIX_COMMON_OP | OP_ESC_LPAREN_SUBEXP |
         OP_ESC_BRACE_INTERVAL ),
 
@@ -475,6 +503,8 @@ public final class Syntax implements SyntaxProperties{
 
         0,
 
+        0,
+
         ( Option.SINGLELINE | Option.MULTILINE ),
 
         new MetaCharTable(
@@ -488,12 +518,15 @@ public final class Syntax implements SyntaxProperties{
     );
 
     public static final Syntax PosixExtended = new Syntax(
+        "PosixExtended",
         ( POSIX_COMMON_OP | OP_LPAREN_SUBEXP |
         OP_BRACE_INTERVAL |
         OP_PLUS_ONE_INF | OP_QMARK_ZERO_ONE |OP_VBAR_ALT ),
 
         0,
 
+        0,
+
         ( CONTEXT_INDEP_ANCHORS |
         CONTEXT_INDEP_REPEAT_OPS | CONTEXT_INVALID_REPEAT_OPS |
         ALLOW_UNMATCHED_CLOSE_SUBEXP |
@@ -512,6 +545,7 @@ public final class Syntax implements SyntaxProperties{
     );
 
     public static final Syntax Emacs = new Syntax(
+        "Emacs",
         ( OP_DOT_ANYCHAR | OP_BRACKET_CC |
         OP_ESC_BRACE_INTERVAL |
         OP_ESC_LPAREN_SUBEXP | OP_ESC_VBAR_ALT |
@@ -521,6 +555,8 @@ public final class Syntax implements SyntaxProperties{
 
         OP2_ESC_GNU_BUF_ANCHOR,
 
+        0,
+
         ALLOW_EMPTY_RANGE_IN_CC,
 
         Option.NONE,
@@ -536,6 +572,7 @@ public final class Syntax implements SyntaxProperties{
     );
 
     public static final Syntax Grep = new Syntax(
+        "Grep",
         ( OP_DOT_ANYCHAR | OP_BRACKET_CC | OP_POSIX_BRACKET |
         OP_ESC_BRACE_INTERVAL | OP_ESC_LPAREN_SUBEXP |
         OP_ESC_VBAR_ALT |
@@ -546,6 +583,8 @@ public final class Syntax implements SyntaxProperties{
 
         0,
 
+        0,
+
         ( ALLOW_EMPTY_RANGE_IN_CC | NOT_NEWLINE_IN_NEGATIVE_CC ),
 
         Option.NONE,
@@ -561,8 +600,13 @@ public final class Syntax implements SyntaxProperties{
     );
 
     public static final Syntax GnuRegex = new Syntax(
+        "GnuRegex",
         GNU_REGEX_OP,
+
         0,
+
+        0,
+
         GNU_REGEX_BV,
 
         Option.NONE,
@@ -578,6 +622,7 @@ public final class Syntax implements SyntaxProperties{
     );
 
     public static final Syntax Java = new Syntax(
+        "Java",
         (( GNU_REGEX_OP | OP_QMARK_NON_GREEDY |
         OP_ESC_CONTROL_CHARS | OP_ESC_C_CONTROL |
         OP_ESC_OCTAL3 | OP_ESC_X_HEX2 )
@@ -586,12 +631,15 @@ public final class Syntax implements SyntaxProperties{
         ( OP2_ESC_CAPITAL_Q_QUOTE | OP2_QMARK_GROUP_EFFECT |
         OP2_OPTION_PERL | OP2_PLUS_POSSESSIVE_REPEAT |
         OP2_PLUS_POSSESSIVE_INTERVAL | OP2_CCLASS_SET_OP |
+        OP2_QMARK_LT_NAMED_GROUP | OP2_ESC_K_NAMED_BACKREF |
         OP2_ESC_V_VTAB | OP2_ESC_U_HEX4 |
         OP2_ESC_P_BRACE_CHAR_PROPERTY ),
 
+        0,
+
         ( GNU_REGEX_BV | DIFFERENT_LEN_ALT_LOOK_BEHIND ),
 
-        Option.SINGLELINE,
+        (Option.SINGLELINE | Option.WORD_BOUND_ALL_RANGE | Option.WORD_BOUND_ALL_RANGE),
 
         new MetaCharTable(
             '\\',                          /* esc */
@@ -604,6 +652,7 @@ public final class Syntax implements SyntaxProperties{
     );
 
     public static final Syntax Perl = new Syntax(
+        "Perl",
         (( GNU_REGEX_OP | OP_QMARK_NON_GREEDY |
         OP_ESC_OCTAL3 | OP_ESC_X_HEX2 |
         OP_ESC_X_BRACE_HEX8 | OP_ESC_CONTROL_CHARS |
@@ -615,6 +664,8 @@ public final class Syntax implements SyntaxProperties{
         OP2_ESC_P_BRACE_CHAR_PROPERTY |
         OP2_ESC_P_BRACE_CIRCUMFLEX_NOT ),
 
+        0,
+
         GNU_REGEX_BV,
 
         Option.SINGLELINE,
@@ -630,6 +681,7 @@ public final class Syntax implements SyntaxProperties{
     );
 
     public static final Syntax PerlNG = new Syntax(
+        "PerlNG",
         (( GNU_REGEX_OP | OP_QMARK_NON_GREEDY |
         OP_ESC_OCTAL3 | OP_ESC_X_HEX2 |
         OP_ESC_X_BRACE_HEX8 | OP_ESC_CONTROL_CHARS |
@@ -644,6 +696,8 @@ public final class Syntax implements SyntaxProperties{
         OP2_ESC_K_NAMED_BACKREF        |
         OP2_ESC_G_SUBEXP_CALL ),
 
+        0,
+
         ( GNU_REGEX_BV |
         CAPTURE_ONLY_NAMED_GROUP |
         ALLOW_MULTIPLEX_DEFINITION_NAME ),
@@ -661,6 +715,7 @@ public final class Syntax implements SyntaxProperties{
     );
 
     public static final Syntax ECMAScript = new Syntax(
+        "ECMAScript",
         (( GNU_REGEX_OP | OP_QMARK_NON_GREEDY |
         OP_ESC_OCTAL3 | OP_ESC_X_HEX2 |
         OP_ESC_CONTROL_CHARS | OP_ESC_C_CONTROL |
@@ -672,14 +727,15 @@ public final class Syntax implements SyntaxProperties{
         OP2_QMARK_GROUP_EFFECT | OP2_OPTION_PERL |
         OP2_ESC_P_BRACE_CHAR_PROPERTY |
         OP2_ESC_P_BRACE_CIRCUMFLEX_NOT |
-        OP2_ESC_U_HEX4 | OP2_ESC_V_VTAB |
-        OP2_OPTION_ECMASCRIPT ),
+        OP2_ESC_U_HEX4 | OP2_ESC_V_VTAB),
+
+        OP3_OPTION_ECMASCRIPT,
 
         ( CONTEXT_INDEP_ANCHORS |
         CONTEXT_INDEP_REPEAT_OPS |
         CONTEXT_INVALID_REPEAT_OPS |
         ALLOW_INVALID_INTERVAL |
-        BACKSLASH_ESCAPE_IN_CC | 
+        BACKSLASH_ESCAPE_IN_CC |
         ALLOW_DOUBLE_RANGE_OP_IN_CC |
         DIFFERENT_LEN_ALT_LOOK_BEHIND ),
 
