@@ -23,23 +23,24 @@ import org.joni.constants.AnchorType;
 
 public final class AnchorNode extends Node {
     public final int type;
-    public boolean asciiRange;
+    public final boolean asciiRange;
     public Node target;
     public int charLength;
 
-    public AnchorNode(int type) {
+    public AnchorNode(int type, boolean asciiRange) {
+        super(ANCHOR);
         this.type = type;
         charLength = -1;
+        this.asciiRange = asciiRange;
+    }
+
+    public AnchorNode(int type) {
+        this(type, false);
     }
 
     @Override
-    public int getType() {
-        return ANCHOR;
-    }
-
-    @Override
-    protected void setChild(Node newChild) {
-        target = newChild;
+    protected void setChild(Node child) {
+        target = child;
     }
 
     @Override
@@ -61,7 +62,7 @@ public final class AnchorNode extends Node {
     public String toString(int level) {
         StringBuilder value = new StringBuilder();
         value.append("\n  type: " + typeToString());
-        value.append("\n  ascii: " + asciiRange);
+        value.append(", ascii: " + asciiRange);
         value.append("\n  target: " + pad(target, level + 1));
         return value.toString();
     }
