@@ -24,8 +24,7 @@ import org.joni.Option;
 import org.joni.constants.EncloseType;
 
 public final class EncloseNode extends StateNode implements EncloseType {
-
-    public int type;                // enclose type
+    public final int type;          // enclose type
     public int regNum;
     public int option;
     public Node target;             /* EncloseNode : ENCLOSE_MEMORY */
@@ -34,35 +33,31 @@ public final class EncloseNode extends StateNode implements EncloseType {
     public int maxLength;           // OnigDistance
     public int charLength;
     public int optCount;            // referenced count in optimize_node_left()
-    public Node containingAnchor;   // 
+    public Node containingAnchor;   //
 
     // node_new_enclose / onig_node_new_enclose
     public EncloseNode(int type) {
+        super(ENCLOSE);
         this.type = type;
         callAddr = -1;
     }
 
-    // node_new_enclose_memory
-    public EncloseNode(int option, boolean isNamed) {
-        this(MEMORY);
-        if (isNamed) setNamedGroup();
-        if (Config.USE_SUBEXP_CALL) this.option = option;
+    public static EncloseNode newMemory(int option, boolean isNamed) {
+        EncloseNode en = new EncloseNode(MEMORY);
+        if (Config.USE_SUBEXP_CALL) en.option = option;
+        if (isNamed) en.setNamedGroup();
+        return en;
     }
 
-    // node_new_option
-    public EncloseNode(int option, int _) {
-        this(OPTION);
-        this.option = option;
+    public static EncloseNode newOption(int option) {
+        EncloseNode en = new EncloseNode(OPTION);
+        en.option = option;
+        return en;
     }
 
     @Override
-    public int getType() {
-        return ENCLOSE;
-    }
-
-    @Override
-    protected void setChild(Node newChild) {
-        target = newChild;
+    protected void setChild(Node child) {
+        target = child;
     }
 
     @Override
@@ -85,14 +80,13 @@ public final class EncloseNode extends StateNode implements EncloseType {
         StringBuilder value = new StringBuilder(super.toString(level));
         value.append("\n  type: " + typeToString());
         value.append("\n  regNum: " + regNum);
-        value.append("\n  option: " + Option.toString(option));
+        value.append(", option: " + Option.toString(option));
+        value.append(", callAddr: " + callAddr);
+        value.append(", minLength: " + minLength);
+        value.append(", maxLength: " + maxLength);
+        value.append(", charLength: " + charLength);
+        value.append(", optCount: " + optCount);
         value.append("\n  target: " + pad(target, level + 1));
-        value.append("\n  callAddr: " + callAddr);
-        value.append("\n  minLength: " + minLength);
-        value.append("\n  maxLength: " + maxLength);
-        value.append("\n  charLength: " + charLength);
-        value.append("\n  optCount: " + optCount);
-
         return value.toString();
     }
 
@@ -102,7 +96,7 @@ public final class EncloseNode extends StateNode implements EncloseType {
         if (isMemory()) types.append("MEMORY ");
         if (isOption()) types.append("OPTION ");
         if (isCondition()) types.append("CONDITION ");
-
+        if (isAbsent()) types.append("ABSENT ");
         return types.toString();
     }
 
@@ -130,4 +124,7 @@ public final class EncloseNode extends StateNode implements EncloseType {
         return (type & STOP_BACKTRACK) != 0;
     }
 
+    public boolean isAbsent() {
+        return (type & ABSENT) != 0;
+    }
 }
