@@ -19,13 +19,20 @@
  */
 package org.joni.ast;
 
+import static org.joni.ast.QuantifierNode.ReduceType.A;
+import static org.joni.ast.QuantifierNode.ReduceType.AQ;
+import static org.joni.ast.QuantifierNode.ReduceType.ASIS;
+import static org.joni.ast.QuantifierNode.ReduceType.DEL;
+import static org.joni.ast.QuantifierNode.ReduceType.PQ_Q;
+import static org.joni.ast.QuantifierNode.ReduceType.P_QQ;
+import static org.joni.ast.QuantifierNode.ReduceType.QQ;
+
 import org.joni.Config;
 import org.joni.ScanEnvironment;
-import org.joni.constants.Reduce;
 import org.joni.constants.TargetInfo;
 
 public final class QuantifierNode extends StateNode {
-
+    public static final int REPEAT_INFINITE = -1;
     public Node target;
     public int lower;
     public int upper;
@@ -41,6 +48,7 @@ public final class QuantifierNode extends StateNode {
     public int  combExpCheckNum;        /* 1,2,3...: check,  0: no check  */
 
     public QuantifierNode(int lower, int upper, boolean byNumber) {
+        super(QTFR);
         this.lower = lower;
         this.upper = upper;
         greedy = true;
@@ -50,13 +58,8 @@ public final class QuantifierNode extends StateNode {
     }
 
     @Override
-    public int getType() {
-        return QTFR;
-    }
-
-    @Override
-    protected void setChild(Node newChild) {
-        target = newChild;
+    protected void setChild(Node child) {
+        target = child;
     }
 
     @Override
@@ -69,13 +72,6 @@ public final class QuantifierNode extends StateNode {
         tgt.parent = this;
     }
 
-    public StringNode convertToString(int flag) {
-        StringNode sn = new StringNode();
-        sn.flag = flag;
-        sn.swap(this);
-        return sn;
-    }
-
     @Override
     public String getName() {
         return "Quantifier";
@@ -84,16 +80,15 @@ public final class QuantifierNode extends StateNode {
     @Override
     public String toString(int level) {
         StringBuilder value = new StringBuilder(super.toString(level));
-        value.append("\n  target: " + pad(target, level + 1));
         value.append("\n  lower: " + lower);
-        value.append("\n  upper: " + upper);
-        value.append("\n  greedy: " + greedy);
-        value.append("\n  targetEmptyInfo: " + targetEmptyInfo);
+        value.append(", upper: " + upper);
+        value.append(", greedy: " + greedy);
+        value.append(", isRefered: " + isRefered);
+        value.append(", targetEmptyInfo: " + targetEmptyInfo);
+        value.append(", combExpCheckNum: " + combExpCheckNum);
         value.append("\n  headExact: " + pad(headExact, level + 1));
         value.append("\n  nextHeadExact: " + pad(nextHeadExact, level + 1));
-        value.append("\n  isRefered: " + isRefered);
-        value.append("\n  combExpCheckNum: " + combExpCheckNum);
-
+        value.append("\n  target: " + pad(target, level + 1));
         return value.toString();
     }
 
@@ -121,33 +116,50 @@ public final class QuantifierNode extends StateNode {
         return -1;
     }
 
-    protected void set(QuantifierNode other) {
+    protected void copy(QuantifierNode other) {
+        state = other.state;
         setTarget(other.target);
         other.target = null;
         lower = other.lower;
         upper = other.upper;
         greedy = other.greedy;
         targetEmptyInfo = other.targetEmptyInfo;
-
-        //setHeadExact(other.headExact);
-        //setNextHeadExact(other.nextHeadExact);
         headExact = other.headExact;
         nextHeadExact = other.nextHeadExact;
         isRefered = other.isRefered;
         combExpCheckNum = other.combExpCheckNum;
     }
 
+    static enum ReduceType {
+        ASIS,       /* as is */
+        DEL,        /* delete parent */
+        A,          /* to '*'    */
+        AQ,         /* to '*?'   */
+        QQ,         /* to '??'   */
+        P_QQ,       /* to '+)??' */
+        PQ_Q,       /* to '+?)?' */
+    }
+
+    static final ReduceType[][]REDUCE_TABLE = {
+      {DEL,     A,      A,      QQ,     AQ,     ASIS}, /* '?'  */
+      {DEL,     DEL,    DEL,    P_QQ,   P_QQ,   DEL},  /* '*'  */
+      {A,       A,      DEL,    ASIS,   P_QQ,   DEL},  /* '+'  */
+      {DEL,     AQ,     AQ,     DEL,    AQ,     AQ},   /* '??' */
+      {DEL,     DEL,    DEL,    DEL,    DEL,    DEL},  /* '*?' */
+      {ASIS,    PQ_Q,   DEL,    AQ,     AQ,     DEL}   /* '+?' */
+    };
+
+
     public void reduceNestedQuantifier(QuantifierNode other) {
         int pnum = popularNum();
         int cnum = other.popularNum();
 
         if (pnum < 0 || cnum < 0) return;
 
-        switch(Reduce.REDUCE_TABLE[cnum][pnum]) {
+        switch(REDUCE_TABLE[cnum][pnum]) {
         case DEL:
             // no need to set the parent here...
-            // swap ?
-            set(other); // *pnode = *cnode; ???
+            copy(other);
             break;
 
         case A:
@@ -199,9 +211,12 @@ public final class QuantifierNode extends StateNode {
         other.target = null; // remove target from reduced quantifier
     }
 
+    static final String PopularQStr[] = new String[] {"?", "*", "+", "??", "*?", "+?"};
+    static final String ReduceQStr[] = new String[] {"", "", "*", "*?", "??", "+ and ??", "+? and ?"};
+
     public int setQuantifier(Node tgt, boolean group, ScanEnvironment env, byte[]bytes, int p, int end) {
         if (lower == 1 && upper == 1) {
-            if (env.syntax.op2OptionECMAScript()) {
+            if (env.syntax.op3OptionECMAScript()) {
                 setTarget(tgt);
             }
             return 1;
@@ -231,20 +246,15 @@ public final class QuantifierNode extends StateNode {
 
             if (Config.USE_WARNING_REDUNDANT_NESTED_REPEAT_OPERATOR) {
                 if (!isByNumber() && !qnt.isByNumber() && env.syntax.warnReduntantNestedRepeat()) {
-                    switch(Reduce.REDUCE_TABLE[targetQNum][nestQNum]) {
+                    switch(REDUCE_TABLE[targetQNum][nestQNum]) {
                     case ASIS:
                         break;
-
                     case DEL:
-                        env.reg.warnings.warn(new String(bytes, p, end) +
-                                " redundant nested repeat operator");
+                        env.warnings.warn("regular expression has redundant nested repeat operator " + PopularQStr[targetQNum] + " /" + new String(bytes, p, end) + "/");
                         break;
-
                     default:
-                        env.reg.warnings.warn(new String(bytes, p, end) +
-                                " nested repeat operator " + Reduce.PopularQStr[targetQNum] +
-                                " and " + Reduce.PopularQStr[nestQNum] + " was replaced with '" +
-                                Reduce.ReduceQStr[Reduce.REDUCE_TABLE[targetQNum][nestQNum].ordinal()] + "'");
+                        env.warnings.warn("nested repeat operator '" + PopularQStr[targetQNum] + "' and '" + PopularQStr[nestQNum] +
+                                "' was replaced with '" + ReduceQStr[REDUCE_TABLE[targetQNum][nestQNum].ordinal()] + "' in regular expression " + "/" + new String(bytes, p, end) + "/");
                     }
                 }
             } // USE_WARNING_REDUNDANT_NESTED_REPEAT_OPERATOR
@@ -269,9 +279,7 @@ public final class QuantifierNode extends StateNode {
         return 0;
     }
 
-    public static final int REPEAT_INFINITE         = -1;
     public static boolean isRepeatInfinite(int n) {
         return n == REPEAT_INFINITE;
     }
-
 }
