@@ -19,30 +19,35 @@
  */
 package org.joni.test;
 
-import org.joni.Option;
-import org.joni.Syntax;
 import org.jcodings.Encoding;
 import org.jcodings.specific.ASCIIEncoding;
+import org.joni.Option;
+import org.joni.Syntax;
+import org.joni.exception.ErrorMessages;
 
 public class TestA extends Test {
-
+	@Override
     public int option() {
         return Option.DEFAULT;
     }
 
+    @Override
     public Encoding encoding() {
         return ASCIIEncoding.INSTANCE;
     }
 
+    @Override
     public String testEncoding() {
         return "iso-8859-2";
     }
 
+    @Override
     public Syntax syntax() {
-        return Syntax.DEFAULT;
+        return Syntax.TEST;
     }
 
-    public void test() throws InterruptedException {
+    @Override
+    public void test() throws Exception {
         x2s("", "", 0, 0);
         x2s("^", "", 0, 0);
         x2s("$", "", 0, 0);
@@ -522,15 +527,20 @@ public class TestA extends Test {
         x2s("((?<x>x)|(?<y>y))(?(<x>)y|x)", "yx", 0, 2);
         ns("((?<x>x)|(?<y>y))(?(<x>)y|x)", "xx");
         ns("((?<x>x)|(?<y>y))(?(<x>)y|x)", "yy");
+        xerrs("(a)?(?<n>b)?(?(1)a)(?(<n>)b)", ErrorMessages.NUMBERED_BACKREF_OR_CALL_NOT_ALLOWED);
+        xerrs("()(?(2))", ErrorMessages.INVALID_BACKREF);
+        xerrs("(?(700000))", ErrorMessages.INVALID_BACKREF);
 
         x2s("\\R", "\n", 0, 1);
         x2s("\\R", "\r", 0, 1);
         x2s("\\R{3}", "\r\r\n\n", 0, 4);
 
         x2s("\\X{5}", "ăăab\n", 0, 5);
-    }
+        x2s("\\X{5}", "あいab\n", 0, 5);
+        x2s("(?<=a).*b", "aab", 1, 3);
 
-    public static void main(String[] args) throws Throwable{
-        new TestA().run();
+        x2s("([.])", ".", 0, 1);
+        x2s("([a])", "a", 0, 1);
+        x2s("([\\w])", "a", 0, 1);
     }
 }
